
c    Copyright 2003 Lars Ottemoller and Jens Havskov
c
c    This file is part of SEISNET.
c
c    SEISNET is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    Foobar is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with Foobar; if not, write to the Free Software
c    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USa

c
c This file contains SEISNET FORTRAN subroutines that are used for QNX Seislog
c type stations.
c

c
c List of subroutine in alphabetical order:
c
c   qnx_cnordic - convert lgs output to Nordic file format
c   update_qnx - update s-file after waveform file transfer
c   update_qnxext - update s-file after extract file is created

c
c Changes:
c
c  28 Nov 2001 lo: added documentation and clean-up
c  05 Dec 2002 lo: create s-file in update_qnxext if s-file not found
c

      subroutine qnx_cnordic(station_name,base,lastevent,par_set,
     &     event_dir,wave_req_dir,det_wav_option)
c
c convert QNX detection list to SEISAN Nordic format
c
c Lars Ottemoeller, Jan 1997.
c

      implicit none

      include 'seidim.inc'
      integer  seiclen      ! character length

c -- station name
      character*5 base
c -- time
      integer hour, minute,sec
      real seconds 
c -- date
      integer year,month,day
c -- date and time of last event
      integer last_hour, last_minute,last_sec
      integer last_year,last_month,last_day
c -- linetype in Nordic format
c    distance indicator
      character*1 linetype, dist
c -- reporting agency
      character*3 agency
c -- line of QUAKE and nordic file resp.
      character*80 line,nordicline,phaseline,idline
      character*90 o_line      ! line with more than 80 chars
c -- top-directory
       character*60 top_directory
c -- index
       integer ind,inds
c -- directory separator character
      character*1 dchar
c -- text strings
       character*80 chr_text_out
c -- number of events
       integer counter
c -- instrument type in Nordic format
       character*1 instout
c -- idnumber
       character*14 idnumber,idtime
c -- counter
       integer i
c -- waveform file line
       character*80 waveline
c time (day,hour,min)
       character*6 time
c length of line
       integer l
c coda length - duration
       integer coda
c  name of S-file
       character*80 sfile
c station name
       character*5 station_name
c number of events
       integer nevent
c file exists
       logical exist
c wav text
       character*42 wav_text
c flag
       logical flag
c qnx parameter set
       character*1 par_set

       character*40 lastevent ! last event in data base
       character*40 currentevent ! current event in data base
       integer      stat     ! flag to compare date of events
       integer  length       ! length of sfile nam,e
       character*(*) event_dir,wave_req_dir
       integer det_wav_option
       character*240 long_text
       character*14 last_detection

      nevent=0
      last_detection = ' '
c
c get directory structure
c

      call topdir(top_directory)

      ind=seiclen(top_directory)
      inds=seiclen(station_name)
      if (inds.eq.-1) inds=4

      call dir_char(dchar)         ! dirctory delimiter character

c
c open files
c
      call file_ready(station_name(1:inds)//'.INP')
      open(50,file=station_name(1:inds)//'.INP',status='old')
      ind=seiclen(chr_text_out)

c
c set default parameters
c
      agency=station_name(1:3) 
      linetype= '1'

      counter=0


c
c get date and time of last event
c
      read(lastevent(1:4),'(i4)') last_year
      read(lastevent(5:6),'(i2)') last_month
      read(lastevent(7:8),'(i2)') last_day
      read(lastevent(9:10),'(i2)') last_hour
      read(lastevent(11:12),'(i2)') last_minute
      read(lastevent(13:14),'(i2)') last_sec
      write(last_detection,'(i4,5i2)') last_year,last_month,
     &  last_day,last_hour,last_minute,last_sec
      do i=1,14
        if (last_detection(i:i).eq.' ') last_detection(i:i)='0'
      enddo
c     write(*,'(a8,1x,6(i2))') 'last   :',last_year,last_month,
c    *last_day,last_hour,last_minute,last_sec

  10  continue
c
c read original line 
c
      read(50,'(a)',end=900) o_line

c
c  find length of line, subtract 80 to get number of characters that shift 
c  text to the right
c
      l = seiclen(o_line)-75

c
c   read line from o_line to get 80 characters
c

      if (l.lt.0) goto 20
      if (l.eq.0) then
         line(1:75)=o_line(1:75)
      elseif (l.gt.0.and.o_line(6:6).ne.' ') then
         line(1:75)=o_line(l+1:l+75)
      else
         line(1:seiclen(o_line))=o_line(1:seiclen(o_line))
      endif

c
c  example of line
c200 97 252  9  9  0:54:29.35  34      4       1  1116 9709_09_0053_48T.TRO_03_1
c1199 98  13  1 13  0:48:12.38  70     10       3 9801_13_0047_31T.SUE_06_1
c 609 98 229  8 17 17:25:41.28  518098424 1312795 9808_17_1724_28T.KMY_03_1
c 610 98 229  8 17 17:27:42.27  3410653310 1316255 9808_17_1726_18T.KMY_03_1
c1338 98 243  8 31 23:31:10.78  39     21       2 9808_31_2330_30T.TRO_03_1
c  56 99 111  4 21  9:51:45.81  37    76      7 1999_04_21_0951_05T.RND_03_1

c                            

      if (line(55:55).ne.'_'.and.line(56:56).ne.'_') THEN
        GOTO 20 
      endif

      counter=counter+1

      wav_text = '                                          '
c
c put waveform name
c

c
c Seislog version 7.9
c
      if (line(48:50).eq.' 19'.or.line(48:50).eq.' 20') then
        wav_text(1:28) = line(49:76)

c
c Seislog before 7.9
c
      elseif (line(50:50).eq.' ') then
        wav_text(1:27)=line(51:75)   
      else
        wav_text(1:27)=line(52:76)
      endif

c
c check if correct parameter set
c
      if (wav_text(seiclen(wav_text):seiclen(wav_text)).ne.par_set) then
c         write(*,'(a)') ' check number of parameter set ! '
         goto 10
      endif
  

      call write_waveline(waveline,'EVE',station_name,'QNX',
     &     wav_text)


c200 97 252  9  9  0:54:29.35
c1199 98  13  1 13  0:48:12.38
      read(line(7:8),'(i2)') year
c
c  add century
c
      if (year.gt.50) then
         year = year + 1900
      else
         year = year + 2000
      endif
      read(line(14:15),'(i2)') month
      read(line(17:18),'(i2)') day

c
c   bug on QNX gives wrong day
c
      if (day.gt.31) then
        read(line(56:57),'(i2)')  day
        if (day.gt.31) then
          write(*,*) 'wrong day!'
          goto 10
        endif
      endif

      read(line(20:21),'(i2)') hour
      read(line(23:24),'(i2)') minute
      read(line(26:30),'(f5.2)') seconds

c  read signal duration
      read(line(31:34),'(i4)') coda
 
      sec= abs(seconds)

      write(currentevent,'(i4,5(i2))') year,month,day,hour,
     *minute,sec
      do i=1,14
         if (currentevent(i:i).eq.' ') then
           currentevent(i:i)= '0'
         endif
      enddo
c      write(*,'(a8,1x,6(i2))') 'current:',year,month,day,hour,
c     *minute,sec


c
c decide to convert or not
c if date of current is newer (greater) than date of last event
c then convert it, otherwise, bypass
c
      call compare(currentevent,lastevent,stat)
      if (stat.le.0) goto 25

  15  continue
c     write(*,*) 'CONVERTING ...'

c set time of last detection
      last_detection=currentevent

c      year= year + 1900
      instout='S'
      dist= 'R'
      
c
c write nordic line
c
      nordicline= ' '

      write(nordicline(2:5),'(i4)') year
      write(nordicline(7:8),'(i2)') month
      write(nordicline(9:10),'(i2)') day   
      write(nordicline(12:13),'(i2)') hour
      write(nordicline(14:15),'(i2)') minute
      write(nordicline(17:18),'(i2)') sec
      write(nordicline(22:22),'(a1)') dist
c      write(nordicline(45:45),'(a1)') '*'
      write(nordicline(46:48),'(a3)') agency
      write(nordicline(51:51),'(a1)') '1'
      write(nordicline(80:80),'(a1)') linetype

c
c  write idline
c
      call write_idline(idline,year,month,day,hour,minute,sec)

      phaseline=' '
      write(phaseline(2:6),'(a5)') station_name
      write(phaseline(7:7),'(a1)') instout
      write(phaseline(8:8),'(a1)') 'Z'
      write(phaseline(10:11),'(a2)') 'EP'
      write(phaseline(16:16),'(a1)') 'A' 
      write(phaseline(19:20),'(i2)') hour
      write(phaseline(21:22),'(i2)') minute
      write(phaseline(24:28),'(f5.2)') seconds
      write(phaseline(30:33),'(i4)') coda

c
c open output file
c

c      if (year.gt.1900) year=year-1900

      call sfilname(year,month,day,hour,
     &      minute,sec,base,dist,sfile,length )

c
c  skip if file exists
c
      inquire(file=sfile,exist=exist)

      if (exist) goto 20

      call file_ready(sfile)

      open(60,file=sfile,status='new')
c
c output
c
      write(*,'(a)') nordicline
      write(60,'(a80)') nordicline
      write(60,'(a80)') idline
      write(60,'(a80)') waveline
      write(60,'(2a40)') ' STAT SP IPHASW D HRMM SECON CODA AMPLIT',
     &   ' PERI AZIMU VELO SNR AR TRES W  DIS CAZ7'
c      write(60,'(a80)') phaseline
      write(60,*)

      close(60)

c
c copy sfile to event directory, if waveform transfer to central base
c    end event directory set
c

      call copy_det_file(sfile,event_dir,wave_req_dir,det_wav_option)

c        long_text='cp '//sfile(1:seiclen(sfile))//' '//
c     &        event_dir
c        call systemc(long_text,seiclen(long_text))
c      endif

      nevent=nevent + 1

  20  CONTINUE
      goto 10

  25  continue
      goto 10

 900  continue

      close(50)
c      close(60)

      write(*,*) '... in total ',nevent,' detections ...'
      write(*,*) '... last detection ',last_detection,' ...'
  
      return
      end    ! end of qnx_cnordic


      subroutine update_qnx(S_file,wave_qnx,wave_name,autopick)
c
c update S-file after requested waveform file has been transferred
c
      implicit none

      include 'libsei.inc'                ! Open file definitions
      include 'seidim.inc'

      character*255 S_file              ! name of S-file
      character*40  wave_name,wave_qnx  ! name of waveform file
      character*8 autopick              ! name of autopick routine

      logical      there                ! test if file exists
 
c-- event data
       character*80     data(max_data)
c-- exp indicator
       character*1      exp
c-- event type
       character*1      type
c-- number of recors for event
       integer          nrecord
c-- number of header lines
       integer          nhead
c number of phases 
       integer nphase
c read unit
       integer read01
c number of stations
       integer nstat
c-- id line number
       integer id
c counter
      integer x
c line with 'REQUEST'
      integer requ_line
c waveform line
      character*80 waveline
c autopick line
      character*80 autopick_line
c index
      integer ind
c flag if autopick line
      logical flag
      integer seiclen


c
c   check if file exists
c
      inquire (file=S_file,exist=there)
      if (.NOT.THERE) then
         write(*,*) S_file,' does not exist !'
         return
      endif


c
c  open S-file
c
      call file_ready(S_file)
      open(unit=8,file=S_file,status='old')

c
c read event file
c
      read01=8
      call indata(read01,nstat,nphase,nhead,nrecord,type,exp,data,id)

      close(8)

c
c look for "REQUEST"
c
      requ_line=-1
      do x=1,nhead
        if (data(x)(66:72).eq.'REQUEST'.or.
     &      data(x)(76:78).eq.'REQ') then     ! changed April 2001, lo
          if (data(x)(23:47).eq.wave_qnx(1:25)) then 
            requ_line=x
          endif
        endif
      enddo
      if (requ_line.eq.-1) then
        write(*,*) ' no request in s-file ... '
        return
      endif

c
c  make autopick_line, use write_autopick_line new April 2001, lo
c
cNET WAV EVE TRO  QNX 9809_05_0000_58T.TRO_03_1                  AUTOPICK      

      call write_autopick_line(data(requ_line),autopick_line,
     &    wave_name,autopick,flag)

c
c update line
c
c        data(requ_line)(66:76) = 'TRANSFERRED'
c        if (never) then
c          data(requ_line)(76:78) = 'NEV' 
c        else
          data(requ_line)(76:78) = 'TRA'   ! changed April 2001, lo
c        endif

c
c blank new line
c
        do x=1,80
           waveline(x:x) = ' '
        enddo
c
c write info to new line
c
        write(waveline(2:41),'(a40)') wave_name
        write(waveline(80:80),'(a1)') '6'
c
c update according to status of file transfer
c
        call file_ready(S_file)
        open(8,file=S_file,status='unknown')

        do x=1,nhead-1
           write(8,'(a80)') data(x)
        enddo

        if (flag) write(8,'(a80)') autopick_line
        write(8,'(a80)') waveline

        do x=nhead,nrecord
          write(8,'(a80)') data(x)
        enddo
c
c close updated S-file and end process
c

        close(8)
      return
      end



      subroutine update_qnxext(S_file,station,wave_qnx,wave_name,
     &     autopick,wave_req_dir)

c
c update s-file after extract file is created
c
      implicit none

      include 'libsei.inc'                ! Open file definitions
      include 'seidim.inc'

      character*255 S_file              ! name of S-file
      character*40  wave_qnx  ! name of waveform file
      character*50 wave_name
      character*8 autopick              ! name of autopick routine
      character*(*) wave_req_dir

      logical      there                ! test if file exists

c-- event data
       character*80     data(max_data)
c-- exp indicator
       character*1      exp
c-- event type
       character*1      type
c-- number of recors for event
       integer          nrecord
c-- number of header lines
       integer          nhead
c number of phases
       integer nphase
c read unit
       integer read01
c number of stations
       integer nstat
c-- id line number
       integer id
c counter
      integer x,y
c line with 'REQUEST'
      integer requ_line
c waveform line
      character*80 waveline,idline,nordicline
      character*42 wav_text
c autopick line
      character*80 autopick_line
c flag if autopick line
      logical flag
c index
      integer ind
      integer seiclen
c station name
      character*5 station
c flag for never transfer file
      logical never
c delete flag
      logical del_flag
c waveform request file
      character*240 reqfile
      integer year,month,day,hour,min,sec


c
c check if wave_name is set to never, which means that data cannot be transferred
c
      never=.false. 
      if (wave_name.eq.'never') never=.true.

c
c remove path from wave_name
c
      y=-1
      do x=seiclen(wave_name),1,-1
        if (wave_name(x:x).eq.'/'.and.y.eq.-1) then
          y=x
        endif
      enddo

      wave_name = wave_name(y+1:seiclen(wave_name))

c
c   check if file exists
c
      inquire (file=S_file,exist=there)
      if (.NOT.THERE) then
         write(*,*) S_file(1:seiclen(s_file)),' does not exist !'
         write(*,*) ' creating new file ... '
c
c create s-file
c
        call file_ready(S_file)
        open(unit=8,file=S_file,status='new')

        read(s_file(seiclen(s_file)-18:seiclen(s_file)),
     &       '(i2,1x,i2,i2,1x,i2,3x,i4,i2)')
     &       day,hour,min,sec,year,month
        wav_text=wave_qnx
        call write_waveline(waveline,'EXT',station,'QNX',
     &          wave_qnx)
        call write_idline(idline,year,month,day,hour,min,sec)
        call write_nordicline(nordicline,year,month,day,hour,min,
     &     float(sec),'R',0.,0.,0.,'1',station(1:3),1)
        write(8,'(a80)') nordicline
        write(8,'(a80)') waveline
        write(8,'(a80)') idline
        write(8,'(2a40)') ' STAT SP IPHASW D HRMM SECON CODA AMPLIT',
     &   ' PERI AZIMU VELO SNR AR TRES W  DIS CAZ7'
        write(8,'(a)')
        close(8)
      endif

c
c  open S-file
c
      call file_ready(S_file)
      open(unit=8,file=S_file,status='old')

c
c read event file
c
      read01=8
      call indata(read01,nstat,nphase,nhead,nrecord,type,exp,data,id)

      close(8)

c
c look for "REQUEST"
cNET WAV EXT RUND QNX 99  8 22 10  0  0                          REQUEST       3
c
      do x=1,nhead
        if (data(x)(66:72).eq.'REQUEST'.or.
     &      data(x)(76:78).eq.'REQ') then
          if (data(x)(23:39).eq.wave_qnx(1:17)) then
            if (data(x)(14:18).eq.station(1:5)) then
              requ_line=x
            endif
          endif
        endif
      enddo

c
c update line
c
        if (never) then
          data(requ_line)(76:78) = 'NEV' 
        else

          data(requ_line)(10:12) = 'EVE'
c
c write waveform name to request line
c
          write(data(requ_line)(23:64),'(a42)') wave_name
       
          reqfile=' '
          reqfile=wave_req_dir(1:seiclen(wave_req_dir))//'/'//
     &       station(1:seiclen(station))//'.wav'
          call freefile(reqfile)
          open(8,file=reqfile,status='unknown')
          write(8,'(a,1x,a)') wave_name,s_file
          close(8)
        endif
c
c if never check if type '6' or request, otherwise delete
c 
        del_flag=.true.
        if (never) then
          do x=1,nhead
            if (data(x)(80:80).eq.'6'.or.
     &          data(x)(76:78).eq.'TRA'.or.
     &          data(x)(76:78).eq.'REQ') then
              del_flag=.false.
            endif
          enddo 
        endif

c
c update according to status of file transfer
c
        call file_ready(S_file)
        open(8,file=s_file,status='unknown')

        do x=1,nhead-1
           write(8,'(a80)') data(x)
        enddo

        do x=nhead,nrecord
          write(8,'(a80)') data(x)
        enddo
c
c close updated S-file and end process
c
        if (never.and.del_flag) then
          close(8,STATUS='DELETE')
        else
          close(8)
        endif


      return
      end



