
c    Copyright 2003 Lars Ottemoller and Jens Havskov
c
c    This file is part of SEISNET.
c
c    SEISNET is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    Foobar is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with Foobar; if not, write to the Free Software
c    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USa

c
c This file contains SEISNET FORTRAN subroutines that are used for SDAS 
c type stations.
c

c
c List of subroutine in alphabetical order:
c
c   sdas_cnordic - convert event list to Nordic file format
c   update_sdas  - update sfile after waveform transfer

      subroutine sdas_cnordic(station_name,base,lastevent,
     &  event_dir,wave_req_dir,det_wav_option,pre_event_time,
     &  event_duration,par_set)
 
c
c sdas_cnordic: convert event list to Nordic file format
c Lars Ottemoeller, Nov 2002
c

      implicit none

      include 'seidim.inc'
      integer  seiclen      ! character length

c -- station name
      character*5 base
c -- parameter set
      character*1 par_set
c -- time
      integer hour, minute,sec
      real seconds
c -- date
      integer year,month,day,doy
c -- pre event time and duration
      real pre_event_time,event_duration
c -- date and time of last event
      integer last_hour, last_minute,last_sec
      integer last_year,last_month,last_day
c -- linetype in Nordic format
c    distance indicator
      character*1 linetype, dist
c -- reporting agency
      character*3 agency
c -- line of QUAKE and nordic file resp.
      character*80 line,nordicline,phaseline,idline
c -- top-directory
      character*60 top_directory
c -- index
      integer ind,inds
c -- directory separator character
      character*1 dchar
c -- text strings
      character*80 chr_text_out
c -- number of events
      integer counter
c -- instrument type in Nordic format
      character*1 instout
c -- idnumber
      character*14 idnumber,idtime
c -- counter
      integer i
c -- waveform file line
      character*80 waveline
c time (day,hour,min)
      character*6 time
c length of line
      integer l
c coda length - duration
      integer coda
c  name of S-file
      character*80 sfile
c station name
      character*5 station_name
c number of events
      integer nevent
c file exists
      logical exist
c wav text
      character*42 wav_text
c flag
      logical flag
c qnx parameter set
      character*1 par_set

      character*40 lastevent ! last event in data base
      character*40 currentevent ! current event in data base
      integer      stat     ! flag to compare date of events
      integer  length       ! length of sfile nam,e
      character*(*) event_dir,wave_req_dir
      integer det_wav_option
      character*240 long_text
      character*14 last_detection
      double precision msec1,msec2  ! time in seconds

      nevent=0
      last_detection = ' '

c
c get directory structure
c
      call topdir(top_directory)
      inds=seiclen(station_name)
      if (inds.eq.-1) inds=4

      call dir_char(dchar)         ! dirctory delimiter character

c
c open files
c
      call file_ready(station_name(1:inds)//'.INP')
      open(50,file=station_name(1:inds)//'.INP',status='old')
      ind=seiclen(chr_text_out)
c
c read 1 line
c
      read(50,'(a)',end=900) line 

c
c set default parameters
c
      agency=station_name(1:3)
      linetype= '1'

      counter=0
c      read(lastevent,'(i2,1x,i2,1x,i4,1x,i2,1x,i2,1x,i2)') 
c     &   last_day,last_month,last_year,last_hour,last_minute,last_sec
      write(last_detection,'(i4.4,5i2.2)') last_year,last_month,
     &  last_day,last_hour,last_minute,last_sec

  10  continue

      read(50,'(a)',end=900) line 
      if (line(3:3).ne.'-') goto 900

c 12-11-2002 08:59:38.000      
      read(line,'(i2,1x,i2,1x,i4,1x,i2,1x,i2,1x,f6.3)') 
     &    day,month,year,hour,minute,seconds

c
c check for parameter set
c
c2-10-2003 11:20:48.000      1.000 ltasta 0
      if (par_set.ne.'?'.and.par_set.ne.line(43:43)) goto 10
      if (line(45:45).eq.'0') goto 10   ! require at least one station
 
      call timsec(year,month,day,hour,minute,seconds,msec1)
      msec1=msec1-pre_event_time 
     
      sec=int(seconds)
      dist='R'
      sfile=' '
      call sfilname(year,month,day,hour,
     &      minute,sec,base,dist,sfile,length )
      write(*,*) sfile(1:seiclen(sfile))
c
c  skip if file exists
c
      inquire(file=sfile,exist=exist)
      if (exist) goto 10
 
      nordicline=' '
      call write_nordicline(nordicline,year,month,day,hour,minute,
     &    seconds,dist,0.,0.,0.,' ',agency,1)
 
      idline=' '
      call write_idline(idline,year,month,day,hour,minute,sec)

c
c write wave extract line
c
      call sectim(msec1,year,doy,month,day,hour,minute,seconds)
      waveline=' '
      write(wav_text,
     & '(i2.2,a1,i2.2,a1,i4,1x,i2.2,a1,i2.2,a1,i2.2,1x,i5)')
     &   day,'/',month,'/',year,hour,':',minute,':',int(seconds),
     &   int(event_duration)
      write(*,*) seconds,int(seconds),int((seconds-int(seconds))*1000.)
      call write_waveline(waveline,'EXT',station_name,'SDA',wav_text)
      
c
c open, write and close
c
      open(60,file=sfile,status='new')
      write(*,'(a)') nordicline
      write(60,'(a80)') nordicline
      write(60,'(a80)') idline
      write(60,'(a80)') waveline
      write(60,'(2a40)') ' STAT SP IPHASW D HRMM SECON CODA AMPLIT',
     &   ' PERI AZIMU VELO SNR AR TRES W  DIS CAZ7'
      write(60,'(a)') ' '
      close(60)
 
c
c copy sfile to event directory, if waveform transfer to central base
c    end event directory set
c
      call copy_det_file(sfile,event_dir,wave_req_dir,det_wav_option)

      nevent=nevent + 1

      goto 10

 900  continue
      close(50)
c
c output
c
      write(*,*) '... in total ',nevent,' detections ...'
      write(*,*) '... last detection ',last_detection,' ...'

      return
      end


      subroutine update_sdas(sfile,sdasdate,sdastime,
     &                   station,wave_name,autopick)
c
c update s-file after waveform transfer
c
      implicit none
      include 'seidim.inc'
      include 'rea.inc'
      character*(*) sfile,sdasdate,sdastime,wave_name,autopick,station
      logical exist,all
      integer seiclen,code,i
      character*80 data(max_phase)

      all=.true.
      
c open sfile if exist
      inquire (file=sfile,exist=exist)
      if (.not.exist) then
        write(*,'(a)') ' sfile does not exist: '//sfile
        return
      endif
      call file_ready(sfile)
      open(8,file=sfile,status='old')

      call rea_event_in(8,all,data,code)
c
c change request line
cNET WAV EXT LNSD SDA 12/11/2002 08:59:32    30                            REQ 3
c
      do i=1,rea_ncomment
        write(*,*) rea_comment(i)(76:78) // ' '//
     &       rea_comment(i)(23:32) // ' '//
     &       rea_comment(i)(34:41) // ' '//
     &       rea_comment(i)(14:17)
        if (rea_comment(i)(76:78).eq.'REQ'.and.
     &      rea_comment(i)(23:32).eq.sdasdate.and.
     &      rea_comment(i)(34:41).eq.sdastime.and.
c     &      rea_comment(i)(14:17).eq.station) then
     &      rea_comment(i)(14:18).eq.station) then

          if (wave_name.eq.'never') then
         rea_comment(i)(23:78)=

     &   '                                                     NEV' 
           else
         rea_comment(i)(23:78)=
     &   '                                                     TRA' 
         rea_comment(i)(23:73)=wave_name(1:seiclen(wave_name))
           endif

        endif
      enddo

c
c add waveform line
c
      rea_nwav=rea_nwav+1
      rea_wav(rea_nwav)='                                        '//
     &  '                                        ' 
      rea_wav(rea_nwav)(2:)=wave_name(1:seiclen(wave_name))
      rea_wav(rea_nwav)(80:80)='6'

      rewind(8)
      call rea_event_out(8,all,data,code)

      close(8)

      return
      end

