
c    Copyright 2003 Lars Ottemoller and Jens Havskov
c
c    This file is part of SEISNET.
c
c    SEISNET is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    Foobar is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with Foobar; if not, write to the Free Software
c    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USa

c
c This file contains SEISNET FORTRAN subroutines that are used for SEISAN 
c type seismic nodes.
c

c
c List of subroutines in alphabetical order:
c
c seisan_cnordic - read seisan file, turn waveform files into requests and
c                  write out sfiles 
c update_seisan - update sfile after waveform data transfer

      subroutine seisan_cnordic(station,database,seisan_file,
     &     event_dir,wave_req_dir,det_wav_option)
c
c read Nordic file, change waveform lines into request and split into database
c
      implicit none
      include 'seidim.inc'
      include 'rea.inc'
      character*(*) station,database,event_dir,wave_req_dir,seisan_file
      integer det_wav_option

      logical all                         ! true: read all data, false: headers
      integer code                        ! error return code
      integer nevent                      ! number of events in file
      integer i                           ! counter
      character*80 data(max_phase)        ! content of parametric event file 
      character*240 sfile
      integer length
      logical exist
      integer year,month,day,hour,min,sec


c
c open input file
c
      open(1,file=seisan_file,status='old')

      all=.true.                  ! read all parameters
      nevent=0                    ! initialize counter

  50  continue
c
c   read all parameters for one event form file unit 1
c
      call rea_event_in(1,all,data,code)
      if (code.eq.1) goto 1000

      do i=1,rea_nwav
        write(*,*) rea_wav(i)
        rea_ncomment=rea_ncomment+1
        rea_comment(rea_ncomment)=
     &     ' NET WAV EVE      SEI                   '//
     &     '                                   REQ 3' 
        write(rea_comment(rea_ncomment)(14:18),'(a5)') station
        write(rea_comment(rea_ncomment)(23:74),'(a)') rea_wav(i)(2:53)
      enddo
      rea_nwav=0
c
c write out sfile
c
      read(rea_id_line(61:74),'(i4,5(i2))') year,month,day,hour,min,sec
      call sfilname(year,month,day,hour,min,sec,database,
     &    data(1)(22:22),sfile,length)
      inquire(file=sfile,exist=exist)
c
c   write out modified event, the array data has also been modified
c
      if (.not.exist) then
        open(2,file=sfile,status='new')
        call rea_event_out(2,all,data,code)
        close(2)
        call copy_det_file(sfile,event_dir,wave_req_dir,det_wav_option)
      endif

      goto 50
1000  continue
      close(1)
      return
      end

      subroutine update_seisan(sfile,autopick,wavfile)
c
c update sfile after waveform data transfer
c
      implicit none
      include 'seidim.inc'
      include 'rea.inc'
      character*(*) sfile,autopick,wavfile
      logical all
      integer i,seiclen,code,j
      character*80 data(max_phase)        ! content of parametric event file 

      all=.true.
c
c open input file
c
      open(1,file=sfile,status='old')
      call rea_event_in(1,all,data,code)
      if (code.eq.1) goto 1000

      j=seiclen(wavfile)
      do i=1,rea_ncomment
c
c check for request
c
        if (rea_comment(i)(76:78).eq.'REQ'.and.
     &    rea_comment(i)(23:22+j).eq.wavfile(1:j)) then
          rea_comment(i)(76:78)='TRA'
          rea_nwav=rea_nwav+1
          rea_wav(rea_nwav)='                                        '
     &                    //'                                       6'
          write(rea_wav(rea_nwav)(2:1+j),'(a)') wavfile(1:j)
        endif
      enddo
      close(1)
c
c write out modified sfile
c
      call file_ready(sfile)
      open(1,file=sfile,status='old')
      call rea_event_out(1,all,data,code)
      close(1)
       
1000  continue

      return
      end



      subroutine update_seiext(S_file,station,wave_sei,wave_name,
     &     wave_req_dir)

c
c update s-file after extract file is created
c
      implicit none

      include 'libsei.inc'                ! Open file definitions
      include 'seidim.inc'

      character*255 S_file              ! name of S-file
      character*40  wave_sei  ! name of waveform file
      character*50 wave_name
      character*8 autopick              ! name of autopick routine
      character*(*) wave_req_dir

      logical      there                ! test if file exists

c-- event data
       character*80     data(max_data)
c-- exp indicator
       character*1      exp
c-- event type
       character*1      type
c-- number of recors for event
       integer          nrecord
c-- number of header lines
       integer          nhead
c number of phases
       integer nphase
c read unit
       integer read01
c number of stations
       integer nstat
c-- id line number
       integer id
c counter
      integer x,y
c line with 'REQUEST'
      integer requ_line
c waveform line
      character*80 waveline,idline,nordicline
      character*42 wav_text
c autopick line
      character*80 autopick_line
c flag if autopick line
      logical flag
c index
      integer ind
      integer seiclen
c station name
      character*5 station
c flag for never transfer file
      logical never
c delete flag
      logical del_flag
c waveform request file
      character*240 reqfile
      integer year,month,day,hour,min,sec


c
c check if wave_name is set to never, which means that data cannot be transferred
c
      never=.false.
      if (wave_name.eq.'never') never=.true.

c
c   check if file exists
c
      inquire (file=S_file,exist=there)
      if (.NOT.THERE) then
         write(*,*) S_file(1:seiclen(s_file)),' does not exist !'
         write(*,*) ' creating new file ... '
c
c create s-file
c
        call file_ready(S_file)
        open(unit=8,file=S_file,status='new')

        read(s_file(seiclen(s_file)-18:seiclen(s_file)),
     &       '(i2,1x,i2,i2,1x,i2,3x,i4,i2)')
     &       day,hour,min,sec,year,month

        wav_text=wave_sei
        call write_waveline(waveline,'EXT',station,'SEI',
     &          wave_sei)
        call write_idline(idline,year,month,day,hour,min,sec)
        call write_nordicline(nordicline,year,month,day,hour,min,
     &     float(sec),'R',0.,0.,0.,'1',station(1:3),1)
        write(8,'(a80)') nordicline
        write(8,'(a80)') waveline
        write(8,'(a80)') idline
        write(8,'(2a40)') ' STAT SP IPHASW D HRMM SECON CODA AMPLIT',
     &   ' PERI AZIMU VELO SNR AR TRES W  DIS CAZ7'
        write(8,'(a)')
        close(8)
      endif

c
c  open S-file
c
      call file_ready(S_file)
      open(unit=8,file=S_file,status='old')

c
c read event file
c
      read01=8
      call indata(read01,nstat,nphase,nhead,nrecord,type,exp,data,id)

      close(8)

c
c look for "REQUEST"
cNET WAV EXT RUND QNX 99  8 22 10  0  0                          REQUEST       3
c
      do x=1,nhead
        if (data(x)(76:78).eq.'REQ') then
          if (data(x)(23:36).eq.wave_sei(1:14)) then
            if (data(x)(14:18).eq.station(1:5)) then
              requ_line=x
            endif
          endif
        endif
      enddo
      write(*,*) ' xxx ',requ_line


c
c update line
c
        if (never) then
          data(requ_line)(76:78) = 'NEV'
        else
          data(requ_line)(10:12) = 'EVE'
c
c write waveform name to request line
c
          write(data(requ_line)(23:64),'(a42)') wave_name

          reqfile=' '
          reqfile=wave_req_dir(1:seiclen(wave_req_dir))//'/'//
     &       station(1:seiclen(station))//'.wav'
          call freefile(reqfile)
          open(8,file=reqfile,status='unknown')
          write(8,'(a,1x,a)') wave_name,s_file
          close(8)
        endif
c
c if never check if type '6' or request, otherwise delete
c
        del_flag=.true.
        if (never) then
          do x=1,nhead
            if (data(x)(80:80).eq.'6'.or.
     &          data(x)(76:78).eq.'TRA'.or.
     &          data(x)(76:78).eq.'REQ') then
              del_flag=.false.
            endif
          enddo
        endif

c
c update according to status of file transfer
c
        call file_ready(S_file)
        open(8,file=s_file,status='unknown')

        do x=1,nhead-1
           write(8,'(a80)') data(x)
        enddo

        do x=nhead,nrecord
          write(8,'(a80)') data(x)
        enddo
c
c close updated S-file and end process
c
        if (never.and.del_flag) then
          close(8,STATUS='DELETE')
        else
          close(8)
        endif

      return
      end


