
c    Copyright 2003 Lars Ottemoller and Jens Havskov
c
c    This file is part of SEISNET.
c
c    SEISNET is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    Foobar is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with Foobar; if not, write to the Free Software
c    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USa

c
c This file contains general SEISNET FORTRAN subroutines.
c

c
c List of subroutine in alphabetical order:
c
c   add_data - add request lines to s-file
c   add_time - add number of seconds to time string and write to screen (do in Expect)
c   casefold - put all characters of the inout string into UPPER CASE letters
c   cat_file - cat ascii file to screen
c   get_components - for given station read components from parameter file
c   compare - compare 2 strings
c   copy_det_file - copy file source file to destination
c   date_to_sec - convert date string into time in seconds since 1900
c   filecheck - check, if the incoming file is an AutoDRM request mail
c   find_file_def - read file definition pattern from seisnet parameter file
c   find_last - find last event in seisan database
c   freeunit - get a free fortran unit
c   get_end_time - write out time of start time + time in seconds (do in Expect)
c   get_stations - read station names from seisnet parameter file
c   gmttim - return gmt time as 14c string
c   hours_back - get difference in hours between 2 time strings
c   id_line_time - get gmt time in id line date-time format
c   log_summary - produce summary file in seisan database in Nordic format and as html file
c   sys_time - returns gmt time as DDHHMM
c   previous_day - write date of previous day on the screen (do in Expect)
c   station_monitor - update station monitor files
c   systim - write system time to string as YYYYMMDDHHMMSS
c   which_station_type - return type of station
c   write_autopick_line - write seisnet autopic type line
c   write_date - read date from s-file name and write out depending on system type
c   write_idline - write nordic file id-line
c   write_nordicline - write line type 1 of nordic file
c   write_waveline - write seisnet waveform request line

c
c Changes:
c
c  28 Nov 2001 lo: added documentation and clean-up
c  13 Mar 2003 lo: moved call to wave_request 
c


      subroutine write_nordicline(nordicline,year,mon,day,hour,min,
     &    sec,dist,lat,lon,depth,lind,agency,nstation)
c
c write line type 1 of nordic file, arguments given
c
      implicit none

      integer year,mon,day,hour,min,nstation
      real sec,lat,lon,depth
      character*1 dist,lind
      character*3 agency
      character*80 nordicline
      integer i

      do i=1,80
        nordicline(i:i)=' '
      enddo

      write(nordicline(2:5),'(i4)') year
      write(nordicline(7:8),'(i2)') mon
      write(nordicline(9:10),'(i2)') day
      write(nordicline(12:13),'(i2)') hour
      write(nordicline(14:15),'(i2)') min
      write(nordicline(17:20),'(f4.1)') sec
      write(nordicline(22:22),'(a1)') dist
      if (lat.ne.0.)write(nordicline(24:30),'(f7.3)') lat
      if (lon.ne.0.)write(nordicline(31:38),'(f8.3)') lon
      if (depth.ne.0.)write(nordicline(39:43),'(f5.1)') depth
      write(nordicline(45:45),'(a1)') lind
      write(nordicline(46:48),'(a3)') agency
      write(nordicline(51:51),'(i1)') nstation
      write(nordicline(80:80),'(a1)') '1'

      return
      end


      subroutine write_waveline(waveline,trans,station,type,
     &      text)
      implicit none
c
c write seisnet waveform request line
c

      character*(*) waveline
      character*(*) trans,type
      character*(*) station
      character*(*) text
      integer seiclen,i

c2345678901234567890123456789012345678901234567890123456789012345678901234567890
cNET WAV EVE MOR8 QNX 9901_04_0200_34T.MOR_03_1        STA051433 TRANSFERRED   3

      waveline=' NET WAV                             ' //
     &   '                                      REQ 3'

      waveline(10:12) = trans
      waveline(14:18) = station
      waveline(19:21) = type
      i=seiclen(text)
      if (text(i:i).eq.'@') text(i:i)=' '
      waveline(23:23+seiclen(text)) = text(1:seiclen(text))

      return
      end


      subroutine write_idline(idline,year,month,day,hour,min,sec)
      implicit none
c
c write nordic file id line
c

      character*80 idline
      integer year,month,day,hour,min,sec,i
      character*14 idtime,idnumber

      write(idnumber,'(i4,i2,i2,i2,i2,i2)')
     *   year,month,day,hour,min,sec

      do i=1,14
        if (idnumber(i:i).eq.' ') idnumber(i:i)='0'
      enddo

c2345678901234567890123456789012345678901234567890123456789012345678901234567890
cACTION:UPD 99-01-14 10:38 OP:alk  STATUS:               ID:19981201002647 L   I

      idtime = ' '
      call id_line_time(idtime)

      idline=' ACTION:NEW                OP:aut  ' //
     &       'STATUS:               ID:                   I'
      idline(13:26)= idtime
      idline(61:74)= idnumber

      return
      end


      subroutine find_file_def (parameter_file,name,def,fdef)
c
c read file definition pattern from seisnet parameter file
c

      implicit none

c station name
      character*5 name,station_name
c station type
c name of SEISNET parameter file
      character*80 parameter_file
c station number
      integer number
c text line
      character*80 line
c read unit  #1
       integer          read01
c logical for existing file or not
       logical          b_old
c returned code
       integer          code
c index
       integer ind
c number of station
       integer x
c definition
       integer def(12)
c filname def
       character*40 fdef

       include 'seidim.inc'
C
C    Seisan library inserts and routines...
C    ======================================
C
       include 'libsei.inc'                ! Open file definitions
C
       external sei open,                  ! Open file routine.
     &          sei close,                 ! Close file routine.
     &          sei code                   ! Error encoder.
C
C    ============= end of list ==========



            call sei open(old$+warn$,            ! Open a unknown status file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    parameter_file,        ! File name
     &                    read01 ,               ! Write unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.

      do x=1,12
        def(x)=0
      enddo
      x=-1
      station_name=' '
      fdef = ' '
      number = -1
 10   CONTINUE

      read(read01,'(a)',END=20) line
c
c   find station names
c
      if (line(5:12).eq.'STATION(') then

         ind=index(line,')')

         read(line(13:ind-1),'(i3)') x
         read(line(31:35),'(a5)') station_name

         if (station_name.eq.name) number=x

c234567890123456789012345678901234567890
cet FILENAMEYEAR(13)          "  1   2"
      elseif (line(5:17).eq.'FILENAMEYEAR(') then

         ind=index(line,')')

         read(line(18:ind-1),'(i3)') x
         if (x.eq.number) then
           read(line(32:38),'(i3,1x,i3)') def(1),def(2)
         endif

c234567890123456789012345678901234567890
cet FILENAMEMONTH(13)         "  3   4"
      elseif (line(5:18).eq.'FILENAMEMONTH(') then

         ind=index(line,')')

         read(line(19:ind-1),'(i3)') x
         if (x.eq.number) then
           read(line(32:38),'(i3,1x,i3)') def(3),def(4)
         endif

c234567890123456789012345678901234567890
cet FILENAMEDAY(13)           "  6   7"
      elseif (line(5:16).eq.'FILENAMEDAY(') then

         ind=index(line,')')

         read(line(17:ind-1),'(i3)') x
         if (x.eq.number) then
           read(line(32:38),'(i3,1x,i3)') def(5),def(6)
         endif

c234567890123456789012345678901234567890
cet FILENAMEJDAY(13)          "  3   5"
      elseif (line(5:17).eq.'FILENAMEJDAY(') then

         ind=index(line,')')

         read(line(18:ind-1),'(i3)') x
         if (x.eq.number) then
           read(line(32:38),'(i3,1x,i3)') def(5),def(6)
c
c  indicate that julian day is used instead of day and month
c
           def(3)=-1 
           def(4)=-1
         endif
c234567890123456789012345678901234567890
cet FILENAMEHOUR(13)
      elseif (line(5:17).eq.'FILENAMEHOUR(') then

         ind=index(line,')')

         read(line(18:ind-1),'(i3)') x
         if (x.eq.number) then
           read(line(32:38),'(i3,1x,i3)') def(7),def(8)
         endif

c234567890123456789012345678901234567890
cet FILENAMEMIN(13)           " 11  12"
      elseif (line(5:16).eq.'FILENAMEMIN(') then

         ind=index(line,')')

         read(line(17:ind-1),'(i3)') x
         if (x.eq.number) then
           read(line(32:38),'(i3,1x,i3)') def(9),def(10)
         endif

c234567890123456789012345678901234567890
cet FILENAMESEC(13)           " 14  15"
      elseif (line(5:16).eq.'FILENAMESEC(') then

         ind=index(line,')')

         read(line(17:ind-1),'(i3)') x
         if (x.eq.number) then
           read(line(32:38),'(i3,1x,i3)') def(11),def(12)
         endif

      elseif (line(5:20).eq.'FILENAMEPATTERN(') then

c23456789012345678901234567890
cet FILENAMEPATTERN(13)       ????_??_????_???.???_??_1                "

         ind=index(line,')')

         read(line(21:ind-1),'(i3)') x
         if (x.eq.number) then
           read(line(31:70),'(a)') fdef
         endif

      endif


      goto 10
  20  continue

      call sei close (close$,read01,code)

      return
      end
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc




      subroutine compare(time1,time2,stat)
c
c compare two time strings
c if 1st string < 2nd string returns -1
c if 1st string = 2nd string returns  0
c if 1st string > 2nd string returns  1
c
      character*40 time1, time2
      integer stat
      character*14 int_time1, int_time2

      read(time1(1:14),'(a14)') int_time1
      read(time2(1:14),'(a14)') int_time2

c      write(*,*) int_time1,' <> ',int_time2
c     write(*,'(a14,1x,a14)') time1,time2
c     write(*,'(i14,1x,i14)') int_time1,int_time2
      if (int_time1.lt.int_time2) then
         stat= -1
      elseif (int_time1.eq.int_time2) then
         stat= 0
      else
         stat= 1
      endif

      return
      end ! compare




      subroutine find_last(lastfile,base,end_t,back,type)
c
c find last event in seisan database
c
      implicit none

      include 'libsei.inc'                ! Open file definitions
      include 'seidim.inc'

       external sei open,                  ! Open file routine.
     &          sei close,                 ! Close file routine.
     &          sei code                   ! Error encoder.
c-- true if file exists (CLK)
       logical          exist
c-- id line number
       integer id
c logical for existing file or not
       logical          b_old
c returned code
       integer          code

      character*5 base                       ! database name     
      character*14 start_time, end_time,end_t ! start and endtime  
      character*10 key                        ! key for findevin
      integer      from_eev                   ! if from eev        
      integer      event_no                   ! event number       
      character*80 evfile,last_file           ! file names          
      integer      fstart, new_month, status  ! variables used by findevin

      integer year,month,day  ! date
      integer back, count     ! number of months to go back
      character*40 lastfile   ! time of last event in database
      character*6  type       ! type of station, for output format

c lines in Nordic file
       character*80     data(max_data)
c-- exp indicator
       character*1      exp
c-- event type
       character*1      ev_type
c-- number of recors for event
       integer          nrecord
c-- number of header lines
       integer          nhead
c-- number of stations
       integer          nstat
c-- number of phases
       integer nphase
      double precision msec   ! time in sec
c date and time
       integer xy,xdo,xm,xd,xho,xmi,xse
       real rsec
       integer x              ! counter
       logical found          ! indicates if on a day event has been found
c read unit #1
       integer          read01

      end_time(1:8)=end_t(1:8)
      end_time(9:14)='000000'


c
c set starttime, begin 'back' days back
c
      read(end_time(1:4),'(i4)') xy
      read(end_time(5:6),'(i2)') xm
      read(end_time(7:8),'(i2)') xd
      read(end_time(9:10),'(i2)') xho
      read(end_time(11:12),'(i2)') xmi
      read(end_time(13:14),'(i2)') xse

      rsec = xse
 
      call timsec(xy,xm,xd,xho,xmi,rsec,msec)
      msec = msec - 86400 * back 
      call sectim(msec,xy,xdo,xm,xd,xho,xmi,rsec)

      xse = int(rsec)
 
      write(start_time(1:4),'(i4)') xy
      write(start_time(5:6),'(i2)') xm
      write(start_time(7:8),'(i2)') xd
      write(start_time(9:10),'(i2)') xho
      write(start_time(11:12),'(i2)') xmi
      write(start_time(13:14),'(i2)') xse
 
      do x=1,14
        if (start_time(x:x).eq.' ') start_time(x:x)='0'
      enddo
c
c  add one day to the end_time
c
      read(end_time(1:4),'(i4)') xy
      read(end_time(5:6),'(i2)') xm
      read(end_time(7:8),'(i2)') xd
      read(end_time(9:10),'(i2)') xho
      read(end_time(11:12),'(i2)') xmi
      read(end_time(13:14),'(i2)') xse
 
      rsec = xse
 
      call timsec(xy,xm,xd,xho,xmi,rsec,msec)
      msec = msec + 86400 
      call sectim(msec,xy,xdo,xm,xd,xho,xmi,rsec)
 
      xse = int(rsec)
 
      write(end_time(1:4),'(i4)') xy
      write(end_time(5:6),'(i2)') xm
      write(end_time(7:8),'(i2)') xd
      write(end_time(9:10),'(i2)') xho
      write(end_time(11:12),'(i2)') xmi
      write(end_time(13:14),'(i2)') xse
 
      do x=1,14
        if (end_time(x:x).eq.' ') end_time(x:x)='0'
      enddo
 
 

c
c  set variables
c
      found = .false.
      count= 1
      back= back-1
      from_eev= 0


      key= '          '

  20  continue

c
c  search for event files
c
      call findevin(base,start_time,end_time,key,from_eev,event_no,
     *     evfile,fstart,new_month,status)

      if(status.eq.0) then      ! file found

c         write(*,*) evfile

c--   (CLK)
         inquire(file=evfile,exist=exist)
c--   (CLK)
         if (exist) then
       
            call file_ready(evfile)

            call sei open(old$+warn$,            ! Open an old file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    evfile,                ! File name
     &                    read01,                ! Read unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.

c
c  read every file to check if logfile
c
        call indata(read01,nstat,nphase,nhead,nrecord,ev_type,
     &exp,data,id)

           if (data(1)(22:23).ne.'LL') then
             if (data(1)(46:48).ne.'LOG') then

               found = .TRUE.       !   file found
               last_file = evfile
           
              endif
           endif
         else 
            write(*,*) evfile,' does not exist !'
         endif

         call sei close (close$,read01,code)
         goto 20     !  continue to find next file

      elseif (status.eq.3.and..not.found) then   ! no more data
c
c no events found in time interval, use fixed start date
c
                read(start_time(1:4),'(i4)') year
                read(start_time(5:6),'(i2)') month
                read(start_time(7:8),'(i2)') day
                if (type.eq.'IRISA'.or.type.eq.'IRISB') then       !   IRIS
                  write(lastfile,'(i4,1x,i2,1x,i2,a9)') year,month,
     *                 day,  ' 00 00 00'
                elseif (type.eq.'SDAS') then
                  write(lastfile,'(i2.2,a1,i2.2,a1,i4.4,1x,a8)')
     &              day,'/',month,'/',year,'00:00:00'
                else
c
c output for QNX, QUAKE, or SEISAN
c
                     lastfile = start_time
                   if (lastfile(3:3).eq.' ') then
                      lastfile(3:3)= '0'
                   endif
                endif
      endif

c
c  find date of last file depending on station type
c
      if (found) call writedate(lastfile,last_file,type)

      write(*,'(a4)') 'LAST'

      if (type.eq.'IRISA'.or.type.eq.'IRISB') then 
        write(*,'(a40)') lastfile(1:40)
      else
        write(*,'(a40)') lastfile
      endif

      return
      end

c end of subroutine find_last
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


      subroutine hours_back(basename,endtime,backdays,
     *                      station_type)
c
c get difference in hours between 2 time strings
c
c input: basename - database name
c        endtime - refernce for time difference
c        backdays - number of days to search database for
c        station_type - system type of station
c output on screen:
c        difference between last event in the database within "backdays" days
c        and endtime, in hours

      implicit none

      character*5  basename       ! input for findevin
      character*14 endtime        ! input for findevin
 
      integer backdays          ! for back on months
      character*40 last           ! last event in database
      character*(*)station_type   ! type of station, for output format
     
      double precision start,end  ! time in sec 
      double precision diff       ! time difference in sec

      integer output,c            ! difference in hours
c
c  get last event in databaseu, format yymmddhhmmss
c
      do c=2,5 
        if (basename(c:c).eq.' ') basename(c:c)='_'
      enddo

      call find_last(last,basename,endtime,backdays,
     *                      station_type)

c
c  convert both times to sec
c
      call date_to_sec(last(1:14),start)
      call date_to_sec(endtime,end)

c
c  get time difference in hours  
c
      diff = end - start
      output = abs (diff / 3600) + 1

      write(*,*) 'LGS'
      write(*,*) output
      return

      end ! hours_back


      subroutine date_to_sec(input,output)
 
c
c read timestring as given from S-files and get the number of seconds
c
c  by Lars Ottemoeller , july 97
 
      implicit none

      character*14 input        ! time in yymmddhhmmss
      double precision output   ! time in sec
 
 
      integer year, month, day,  hour, min, secs
      real sec

      output=0.
 
c
c   read the time
c
      read(input(1:4),'(i2)',err=999) year
      read(input(5:6),'(i2)',err=999) month
      read(input(7:8),'(i2)',err=999) day
      read(input(9:10),'(i2)',err=999) hour
      read(input(11:12),'(i2)',err=999) min
      read(input(13:14),'(i2)',err=999) secs

      sec = secs
 
c
c convert date and time to seconds
c
      call timsec(year,month,day,hour,min,sec,output)
 
999   continue
      return
 
      end ! date_to_sec
 

      subroutine writedate(last_file,filename,type)
c
c converts S-file name (filename) into a valid date (last_file)
c depending on statin type (type)
c 

      implicit none

      integer year, month, day, hour, min, sec
      character*80 filename
      character*6  type
      integer pos            ! Position in string
      character*40 last_file ! output value
      integer      i,j       ! counter
      integer seiclen

      do i=1,40
        last_file(i:i) = ' '
      enddo

c
c read values to change format
c
      pos=-1
      j=seiclen(filename)
      do i=j,1,-1
        if (filename(i:i).eq.'.'.and.pos.eq.-1) then
          pos = i
        endif
      enddo

c7-1359-56R.S199812
      read(filename(pos+2:pos+5),'(i4)') year
      read(filename(pos+6:pos+7),'(i2)') month
      read(filename(pos-11:pos-10),'(i2)') day
      read(filename(pos-8:pos-7),'(i2)') hour
      read(filename(pos-6:pos-5),'(i2)') min
      read(filename(pos-3:pos-2),'(i2)') sec

       if (type(1:4).eq.'IRIS') then
c
c format for IRIS stations
c
c          if (year.ge.2000) then
c            yy=year-2000
c          else
c            yy=year-1900
c          endif
          write(last_file,'(i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,i2)')
     *    year, month, day, hour, min, sec
       elseif (type(1:4).eq.'SDAS') then
          write(last_file,'
     &      (i2.2,a1,i2.2,a1,i4.4,1x,i2.2,a1,i2.2,a1,i2.2)')
     &      day,'-',month,'-',year,hour,':',min,':',sec
       else
c
c format for QNX, QUAKE and SEISAN
c
          write(last_file,'(i4,5(i2))') year, month, day, hour, min, sec
           do i=1,14
              if (last_file(i:i).eq.' ') then
c
c replace blank spaces for 0
c
                last_file(i:i)= '0'
              endif
           enddo
       endif
c
c end of process
c
      return
      end   ! writedate


 
      subroutine sys_time(time)
c
c return gmt time as day, hour and min
c Lars Ottemoeller , July 1997
c
      implicit none
      character*14 time1
      character*6 time

      call gmttim(time1)
      time = '      ' 
      time (1:6) = time1 (6:12)

      return   ! sys_time
      end


      subroutine id_line_time(time)
c
c get gmt time in id line date-time format
c

c 98-05-01 17:09
      implicit none
      character*14 time1
      character*14 time
      character*2 year,month,day,hour,min 
 
      call gmttim(time1)

      time = '  -  -     :  '
      read(time1(3:4),'(a2)') year
      read(time1(5:6),'(a2)') month
      read(time1(7:8),'(a2)') day  
      read(time1(9:10),'(a2)') hour
      read(time1(11:12),'(a2)') min   
 
      write(time(1:2),'(a2)')  year      
      write(time(4:5),'(a2)')  month     
      write(time(7:8),'(a2)')  day        
      write(time(10:11),'(a2)')  hour       
      write(time(13:14),'(a2)')  min         

      return
      end ! id_line_time


      subroutine write_autopick_line(line,autopick_line,
     &    wave_name,autopick,flag)
c
c write seisnet autopic type line
c
      implicit none
      character*(*) line,autopick_line
      character*(*) wave_name
      character*(*) autopick
      logical flag
      integer seiclen,ind

      autopick_line=line
      if (wave_name.eq.'never') return
      autopick_line(23:74)='                                          '
      ind = seiclen(wave_name)
      autopick_line(23:23+ind) = wave_name(1:ind)
      flag=.false.
      if (autopick.eq.'AUTOPIC') then
        flag=.true.
        autopick_line(76:78)= 'API'
      elseif (autopick.eq.'FINDP') then
        flag=.true.
        autopick_line(76:78)= 'FIP'
      elseif (autopick.eq.'AUTOSIG') then
        flag=.true.
        autopick_line(76:78)= 'ASI'   ! changed April 2001, lo
      endif
       
      return
      end


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

 
 
      subroutine log_summary(log_file,log_base,html_dir)
c
c produce summary file in Seisan database in Nordic format and as html file
c
      implicit none

      character*80 log_file            ! path and name of log file
      character*80 html_dir            ! html directory for logsum file
      character*5  log_base            ! name of Log database
      character*80 line                ! text line
      integer max                      ! maximum number of stations in one file
      parameter(max=99)
      character*5 station,station_name(max)     ! station names
      integer station_par(max),station_wav(max) ! number of par and wav proced.
      integer station_det(max),station_req(max),station_tra(max)
      real station_tim(max),station_upt(max),station_gps(max)
      real time    
      integer station_dsp(max)
      logical exist
      integer  station_number          ! number of stations
      integer x,y,z                    ! counter
      logical new_station              ! is station new
      logical station_err(max)
      character*80 norline
      integer year,mon,day,hour,min,sec,length
      character*1 dist
      character*14 p_time
      character*14 proc_time
      character*80 sfile,html_file
      character*60 top_directory
      character*1 dchar
      character*20 xfile
      integer seiclen

c
c  get seisan top directory
c
      call topdir(top_directory)
      call dir_char(dchar)

c
c check if file exists and open
c
      inquire(file=log_file,exist=exist)

      if (exist) then
         call file_ready(log_file)
         open(1,file=log_file,status='old')
      else
         write(*,*) 'file does not exist !'
         goto 99
      endif

      station_number=0

  10  continue

c
c  read log file
c
      read(1,'(a80)',end=99) line


c
c  get station names
c
      new_station=.TRUE.

      if (line(16:18).eq.'PAR'.or.line(16:18).eq.'WAV') then

        station=line(11:15)

        do x=1,station_number
          if (station.eq.station_name(x)) then
            new_station=.FALSE.
          endif
        enddo

        if (new_station) then
          station_number=station_number+1
          station_name(station_number)=station
        endif
      endif

      goto 10


  99  continue

      if (exist) close(1)

      call file_ready(log_file)
      if (exist) open(1,file=log_file,status='old')

      do x=1,station_number

         station_par(x)=0
         station_wav(x)=0
         station_det(x)=0
         station_tim(x)=0
         station_upt(x)=0
         station_gps(x)=0
         station_dsp(x)=0
         station_req(x)=0
         station_tra(x)=0
         station_err(x)=.FALSE.
      enddo

 100  continue

c07:51:46 TRO  PAR 07:52:12   0.0 119.4 100.0 66%  12                          3
c08:38:02 KBS  PAR ------------------------------   2                          3

      read(1,'(a80)',end=199) line


      if (line(16:18).eq.'PAR'.or.line(16:18).eq.'WAV') then

        do x=1,station_number

          if (line(11:15).eq.station_name(x)) then
            if (line(76:78).eq.'***') station_err(x)=.TRUE.
            if (line(16:18).eq.'PAR') then
               if (line(55:56).eq.'  ')
     &         station_par(x)=station_par(x)+1

               if (line(20:20).ne.'-') then

                 if (line(29:33).ne.'     ') then
                   read(line(29:33),'(f5.1)',err=111) time
                   if (abs(time).gt.abs(station_tim(x))) 
     &                   station_tim(x)=time
                 endif

                 if (line(35:39).ne.'     ') then
                   read(line(35:39),'(f5.1)',err=111) station_upt(x)
                 endif

                 if (line(41:45).ne.'     ') then
                   read(line(41:45),'(f5.1)',err=111) station_gps(x)
                 endif

                 if (line(47:48).ne.'  ') then
                   read(line(47:48),'(i2)',err=111) station_dsp(x)
                 endif

                 if (line(50:53).ne.'    ') then
                   read(line(50:53),'(i4)',err=111) y
                   station_det(x)=station_det(x)+y
                 endif

               elseif (line(50:50).eq.'-') then

               else

                 if (line(50:53).ne.'    ') then
                   read(line(50:53),'(i4)',err=111) y
                   station_det(x)=station_det(x)+y
                 endif
            
               endif

            elseif (line(16:18).eq.'WAV') then
 
               if (line(55:56).eq.'  ')
     &            station_wav(x)=station_wav(x)+1

c09:12:25 ASK  WAV -------- TRANSFER:   0   DONE:   0                          3

               if (line(29:37).eq.'TRANSFER:') then
                 read(line(38:41),'(i4)',err=111) y
                 station_req(x)=station_req(x)+y
               endif

               if (line(45:49).eq.'DONE:') then
                 read(line(50:53),'(i4)',err=111) y
                 station_tra(x)=station_tra(x)+y
               endif
            endif
          endif
        enddo
      endif
 111  continue
      goto 100

 199  continue

c
c   close input file
c
      if (exist) close(1)

c
c  write header line
c
      xfile=log_file(seiclen(log_file)-11:seiclen(log_file))
      read(xfile(1:4),'(i4)') year
      read(xfile(5:6),'(i2)') mon
      read(xfile(7:8),'(i2)') day

c
c  open file
c
c 24-0000-00L.S9802

      do z=1,80
        sfile(z:z)=' '
        html_file(z:z)=' '
      enddo

      sec=1
 205  continue 

      hour=0
      min=0
      dist='L'

c
c always overwrite existing daily log file
c
      call sfilname(year,mon,day,hour,
     &      min,sec,log_base,dist,sfile,length )
      html_file = sfile(1:seiclen(sfile))//'.html'

c      inquire(file=sfile,exist=exist)

c      if (.not.exist) then
        call file_ready(sfile)
        open(1,file=sfile,status='unknown')
        open(2,file=html_file,status='unknown')
c        goto 210
c      endif

c      sec=sec+1
c      goto 205

  210 continue

c
c init html file
c
      write(2,'(a)') '<html>'
      write(2,'(a)') '<head>'
      write(2,'(a)') '<title>SEISNET: LogFile</title>'
      write(2,'(a)') '</head>'
      write(2,'(a)') '<body bgcolor=#ffffff>'
      write(2,'(a)') '<body>'
      write(2,'(a)') '<pre>'
      write(2,'(a)') '<font size="2">'

      write(*,*) 'logfile: ',sfile(1:80)
      
      do y=1,80
        norline(y:y)=' '
      enddo

      write(norline(2:5),'(i4)') year
      write(norline(7:10),'(2i2)') mon,day
      norline(12:48)='0000 00.1 LL                      LOG'
      norline(80:80)='1'

c      write(*,*) norline
      write(1,'(a80)') norline
      write(2,'(a)') norline(2:10)

c
c  write info line
c
      norline( 1:40)=' ---------------------------------------'
      norline(41:80)='---------------------------------------3'

      write(1,'(a80)') norline
      write(2,'(a)') '<hr>'
      
      norline( 1:43)=' NET    CON   NOD REQ TRA TIMED STAUP GPSUP'
      norline(44:80)=' DU%                            ERR 3'

c      write(*,*) norline
      write(1,'(a80)') norline
      write(2,'(a)') norline(1:78)

      norline( 1:40)=' ---------------------------------------'
      norline(41:80)='---------------------------------------3'
 
      write(1,'(a80)') norline
      write(2,'(a)') '<hr>'
 
      do x=1,station_number 

        do y=1,80
          norline(y:y)=' '
        enddo


        write(norline(2:6),'(a5)') station_name(x)
        write(norline(7:11),'(i5)') station_par(x)+station_wav(x)
c        write(norline(11:13),'(i3)') station_wav(x)
        write(norline(15:17),'(i3)') station_det(x)
        write(norline(19:21),'(i3)') station_req(x)
        write(norline(23:25),'(i3)') station_tra(x)
        write(norline(27:31),'(f5.1)') station_tim(x)
c        write(norline(33:37),'(f5.1)') station_upt(x)
        write(norline(33:37),'(a5)') '     '
c        write(norline(39:43),'(f5.1)') station_gps(x)
        write(norline(39:43),'(a5)') '     '
        write(norline(45:47),'(i3)') station_dsp(x)    

        if (station_err(x)) write(norline(76:78),'(a3)') '***'
        norline(80:80)='3'

c        write(*,*) norline
        write(1,'(a80)') norline
        write(2,'(a)') norline(1:79)
      enddo

c
c add id line and net aso line      
c
      norline( 1:40)=' ---------------------------------------'
      norline(41:80)='---------------------------------------3'
 
      write(1,'(a80)') norline
      
cACTION:NEW 98-02-25 08:48 OP:ONL  STATUS:               ID:19980224000000     I
cNET ASO YES                                                                   3

      CALL gmttim(PROC_TIME)

      write(p_time,'(a2,a1,a2,a1,a2,1x,a2,a1,a2)') 
     *  PROC_TIME(3:4),'-',PROC_TIME(5:6),'-',
     *  PROC_TIME(7:8),PROC_TIME(9:10),':', PROC_TIME(11:12)

      norline(1:42)=' ACTION:NEW                OP:LOG  STATUS:'
      WRITE(norline(13:26),'(A)') P_TIME
      norline(43:80)='               ID:                   I' 
      write(norline(61:75),'(i4,6I2)')
     *   year,mon,day,0,0,1
      do z=61,74
        if(norline(z:z).eq.' ') norline(z:z)='0'
      enddo


      write(1,'(a80)') norline
c      write(*,*) norline
 
      norline( 1:40)=' NET ASO YES                            '
      norline(41:80)='                                       3'

 
      write(1,'(a80)') norline
c      write(*,*) norline
      write(2,'(a80)') '<hr>'
      write(2,'(a80)') ' '
      write(2,'(a80)') '</body>'
      write(2,'(a80)') '</html>'

      close(1)
      close(2)
c
c copy html file to html directory, possibly on web server
c
      if (html_dir.ne.'NONE') then
        z=4+seiclen(html_dir)+seiclen(html_file)
        call systemc('mv '//html_file(1:seiclen(html_file))//
     &   ' ' // html_dir(1:seiclen(html_dir)),z)
      endif

      return
      end

 
      

      subroutine previous_day (input_date)
c
c write date of previous day to screen
c

      implicit none

      character*8 input_date,pre_day
      integer year,month,day,hour,minute
      real seconds
      integer y,mo,d,h,mi,dy
      real se
      double precision msec
      integer x

      read(input_date,'(i4,i2,i2)') year,month,day
      hour=12
      minute=0 
      seconds=0

      call timsec(year,month,day,hour,minute,seconds,msec)
      msec = msec - 86400     ! previuos day
 
      call sectim(msec,y,dy,mo,d,h,mi,se)

      if (y.lt.100) y=y+1900

      pre_day = '        '
      write(pre_day,'(i4,i2,i2)') y,mo,d

      do x=1,8
        if (pre_day(x:x).eq.' ') pre_day(x:x)='0'
      enddo

      write(*,*) pre_day



      return
      end



      subroutine get_end_time (start_time,time_dif)

      implicit none

      character*17  start_time   ! start time
      character*17  end_time     ! start time + time diff
      character*5   time_dif     ! time diff
      integer             year,month,day,hour,min,sec
      real                rsec
      double precision    msec
      integer             y,mo,dy,da,h,mi
      integer             diff
      
c8/03/10 10:12:14

      read(start_time(1:2),'(i2)') year

      if (year.gt.50) then
        year = year + 1900
      else
        year = year + 2000
      endif

      read(start_time(4:5),'(i2)') month
      read(start_time(7:8),'(i2)') day 
      read(start_time(10:11),'(i2)') hour
      read(start_time(13:14),'(i2)') min 
      read(start_time(16:17),'(i2)') sec 
      rsec = sec

      read(time_dif(1:5),'(i5)') diff

      call timsec(year,month,day,hour,min,rsec,msec)
      msec = msec + diff


      call sectim(msec,y,dy,mo,da,h,mi,rsec)

      sec = abs(rsec)

      if (y.ge.2000) then
         y = y - 2000
      else
         y = y - 1900
      endif

      write(end_time(1:2),'(i2)') y
       if (end_time(1:1).eq.' ') end_time(1:1)='0'
      end_time(3:3)='/'
      write(end_time(4:5),'(i2)') mo
       if (end_time(4:4).eq.' ') end_time(4:4)='0'
      end_time(6:6)='/'
      write(end_time(7:8),'(i2)') da
       if (end_time(7:7).eq.' ') end_time(7:7)='0'
      end_time(9:9)=' '
      write(end_time(10:11),'(i2)') h
       if (end_time(10:10).eq.' ') end_time(10:10)='0'
      end_time(12:12)=':'
      write(end_time(13:14),'(i2)') mi
       if (end_time(13:13).eq.' ') end_time(13:13)='0'
      end_time(15:15)=':'
      write(end_time(16:17),'(i2)') sec
       if (end_time(16:16).eq.' ') end_time(16:16)='0'

      write(*,*) end_time

      return 
      end




      subroutine station_monitor(path,type,station,date,time)
c
c update station monitor files
c

      implicit none

      character*3 type        ! PAR or WAV
      character*5 station     ! station name
      character*8 time
      character*10 date
      logical exist           ! file exist
      character*80 line       ! text line
      character*75 path
      character*80 fname      ! file name
      integer seiclen

cONO1 PAR 15/10/1998 12:05:06       WAV 15/10/1998 12:05:06

      fname = path(1:seiclen(path)) // '/' // station

      write(*,*) 'file : ',fname

      inquire(file=fname,exist=exist)

      if (exist) then

c
c  read file
c

        open(1,file=fname,status='old')
        read(1,'(a80)',err=50) line
        close(1)
      endif

      goto 60

50    exist = .false. 

60    continue

c
c  initialize
c
      if (.not.exist) then
        line = station //
     &  ' PAR 00/00/0000 00:00:00       WAV 00/00/0000 00:00:00'
      endif


      if (type.eq.'PAR') then

        write(line(11:20),'(a10)') date
        write(line(22:29),'(a8)') time
      
      elseif (type.eq.'WAV') then

        write(line(41:50),'(a10)') date
        write(line(52:59),'(a8)') time

      endif

      open(1,file=fname,status='unknown')

      write(1,'(a80)') line
      close(1)

      



      return
      end



      subroutine cat_file(file)
c
c  display ascii file to screen
c

      implicit none

      character*80 file,line
      integer c

      write(*,*)
      c=1

  10  continue

      open(1,file=file,status='unknown')

      read(1,'(a80)',end=999) line

      write(*,*) line

      c=c+1

      goto 10

 999  continue

      write(*,*)

      close(1)

      return
      end
      


      subroutine add_data(parameter_file,cbase,sfile,ext_stat_index,
     &   ext_channel,ext_time,ext_duration,ext_pre,wave_req_dir)
c
c add request lines to s-file
c
c options are: .... (xxx)
c

      implicit none

      integer ext_stat_index
      character*(*) ext_channel
      character*(*) ext_time
      character*(*) wave_req_dir
      real ext_duration
      integer ext_mode

c max
      integer max
      parameter (max=50)
c list of channels that will be used for station
       character*6 det_channels(max)
c name of sfile
       character*(*) sfile
c channel line
       character*42 channel_line(max)
c number of channels in det_channels
       integer n_ch
c id of channel
       integer channel
c -- waveform file line
       character*80 waveline(max),wave_line
c number of comp
       integer n_comp
c name of central database
      character*(*) cbase
c name of SEISNET par file
      character*(*) parameter_file
c model indicator in station file
      character*1 c
c station name
      character*5 sname(max),station_name,stations(max)
c station type
      character*6 station_type(max)
c input of eq lat and lon in degrees
      real deqlat,deqlon,sdeqlat
c hypocenter depth
      real depth
c minimum travel time
      real tmin
c distance station-epicenter
      real dist(max)
c output of mintt (1-OK, 0-station not found)
      integer iout(max)
c origin time
      integer oyear,omonth,oday,ohour,omin
      real osec
      integer iosec
c start of time interval
      integer year(max),doy(max),mo(max),da(max),h(max),mi(max)
      integer syear
      real sec(max)
c number of stations
      integer nstation
c counter
      integer x,i,xx
c pre event memory
      integer pre_event,event_length
c pre event as real
      real ext_pre
c origin time and interval start in sec
      double precision otsec,startsec,minsec(max),dtmin
c text
      character*80 line
       include 'seidim.inc'
c name of desired stream
       character*6 instrument(max)
c distance indicator
       character*1 disti
c length of s_file name
       integer length
c exist file
       logical exist
c line
       character*80 idline,nordicline
c if new sfile
       logical new_sfile
c file exist
       logical there
c  wave text
       character*42 wav_text
       character*50 naqs_wav_text
c seconds as integer
       integer isec(max)
      character*25 chan_list(6)              ! list of channels to transfer
      character*40 naqs_chan_list(50)        ! list of naqs channels to transfer
      character*40 sdas_chan_list(50)        ! list of naqs channels to transfer
      integer pre_time(6),time_window(6)     ! time interval
      integer nchan                          ! number of channels
     
C
C    Seisan library inserts and routines...
C    ======================================
C
       include 'libsei.inc'                ! Open file definitions
       integer code
       integer seiclen
C
       external sei open,                  ! Open file routine.
     &          sei close,                 ! Close file routine.
     &          sei code                   ! Error encoder.
C
C    ============= end of list ==========
c-- event data
       character*80     data(max_data)
c-- exp indicator
       character*1      exp
c-- event type
       character*1      type
c-- number of recors for event
       integer          nrecord
c-- number of header lines
       integer          nhead
c number of phases
       integer nphase
       integer nstat,id
c fixed time, means no epic and OT known
       logical fixed
c yes or no
       character*1 yn
c input line
       character*80 input_line
c if no location fix_time is true and flag if hypocenter info is found
       logical fix_time,found

      common /iris/det_channels,channel_line,n_ch,station_name,channel

      write(*,*)' Central database: ',cbase
      write(*,*)' Parameter file  : ',parameter_file


      pre_event=-1
      event_length=-1
      ext_mode=0

c
c extract mode 1, stat_index and time given
c
      if (ext_stat_index.gt.0.and.ext_time.ne.' ') then
        ext_mode=1
c        pre_event=60
        pre_event=0   ! assume start of extract is given
        if (ext_pre.gt.0.) pre_event=int(ext_pre) ! lo 20011213
        event_length=int(ext_duration)
c
c extract mode 2, stat_index given, but no ext_time
c
      elseif (ext_stat_index.gt.0.and.ext_time.eq.' ') then
        event_length=int(ext_duration)
        pre_event=60
        if (ext_pre.gt.0.) pre_event=int(ext_pre) ! lo 20011213
        write(*,*) ' pre-event: ',pre_event
        ext_mode=2
      endif
c
c for given sfile
c
      if (sfile.ne.'NONE') then

        new_sfile = .false.
c
c get origin time and location from s-file
c
        inquire(file=sfile,exist=there)
        if (there) then
          open(60,file=sfile,status='old')

        call indata(60,nstat,nphase,nhead,nrecord,type,exp,data,id)

          close(60)
     
          read(data(1)(2:43),'(i4,1x,i2,i2,1x,i2,i2,1x,f4.1,1a,2x,
     &      f7.3,f8.3,f5.1)') oyear,omonth,oday,ohour,omin,
     &      osec,c,deqlat,deqlon,depth

          fixed = .false.
          found = .true.
          if (deqlat.eq.0.and.deqlon.eq.0) then
            found = .false.
c
c search for 2nd headers
c
            do x=2,nhead
              if(data(x)(80:80).eq.'1'.and.
     &                              .not.found) then 

                if (data(x)(24:38).ne.
     &        '               ') then

          read(data(x)(2:43),'(i4,1x,i2,i2,1x,i2,i2,1x,f4.1,1a,2x,
     &      f7.3,f8.3,f5.1)') oyear,omonth,oday,ohour,omin,
     &      osec,c,deqlat,deqlon,depth

                  found = .true.
                endif
              endif

            enddo

            if (.not.found) then

              write(*,*) ' S-file does not have a location'
            write(*,*) ' using time in header as fixed time for all the 
     &stations'
              fixed = .TRUE.
            endif
          endif

        else

          write(*,*) 'S-file does not exist: ' // sfile
          return
         
        endif

      else

        new_sfile = .true.

c
c  if no sfile name then create new event and ask for parameters
c
        c=' ' 
        oyear=0
        omonth=0
        oday=0
        ohour=0
        omin=0
        osec=0
        fixed=.true.

        if (ext_mode.eq.0) then
          write(*,*)
          write(*,*) 'Hypocenter Location and origin time known (y/n)?'
          read(*,*) yn

          if (yn.eq.'y'.or.yn.eq.'Y') then
 
            fixed=.false.
            write(*,*)' calculate min travel time for'
            write(*,*)' given origin time and location'
            write(*,*)
            write(*,*)' which model (only for local, default 0):'
            read(*,'(a1)') c
            write(*,*)
 
            write(*,*) 'latitude of epicenter in degrees:'
            read(*,*) deqlat
            write(*,*) 'longitude of epicenter in degrees:'
            read(*,*) deqlon
   
            write(*,'('' Hypocentral depth in km:'')')
            read(*,*) depth 
            write(*,*) 'Origin time:'
 
          else

            write(*,*) ' Fixed start time '
          endif

          write(*,'(a)') 'yyyy mm dd hh mm ss'

          read(*,'(a)') line
          call SEI GET VALUES(6,line,CODE )
             oyear=ARRAY$(1)
             omonth=ARRAY$(2)
             oday=ARRAY$(3)
             ohour=ARRAY$(4)
             omin=ARRAY$(5)
             osec=ARRAY$(6)

        else
          read(ext_time,'(i4,4i2)') oyear,omonth,oday,ohour,omin
          osec=0.
        endif
        write(*,*) oyear,omonth,oday,ohour,omin,osec
      endif

      call timsec(oyear,omonth,oday,ohour,omin,osec,otsec)
c
c  subtract preevent memory
c
      startsec = otsec 

      do x=1,max
        sname(x)='     '
        station_type(x)='      '
      enddo

c
c read station codes from parameter file
c
      call get_stations(parameter_file,stations,nstation)

      if (ext_mode.eq.0) then
        write(*,*)
        write(*,*) 'station list'
        do x=1,nstation,4
           write(*,'(4(i3,1a,1x,a5))') x,'.',stations(x),
     &      x+1,'.',stations(x+1),x+2,'.',stations(x+2),
     &      x+3,'.',stations(x+3)
        enddo

       write(*,*)' Give numbers to select stations, seperated by " "'//
     &   ', or "all", not more then 15'
        read(*,'(a80)') input_line

        write(*,*) ' Give duration in seconds: (return for default) '
        read(5,'(i)') event_length 
        if (event_length.eq.0) event_length=-1
        do x=1,15
          ARRAY$(x)=0
        enddo

        if (yn.eq.'y'.or.yn.eq.'Y'.or.sfile.ne.'NONE') then
          write(*,*) ' Give pre-event time in seconds: '
          read(5,'(i)') pre_event
          event_length=event_length+pre_event
        else     ! no pre_event time if fixed start time
          pre_event=0
        endif

        if (input_line(1:3).eq.'all'.or.input_line(1:3).eq.'ALL') then
          do x=1,nstation
            ARRAY$(x)=x
          enddo
        else
          call SEI GET VALUES(15,input_line,CODE )
        endif
        nstation=0
        do x=1,15
          if (ARRAY$(x).ne.0) then
            xx=ARRAY$(x)
            if (stations(xx).ne.''.and.
     &         stations(xx).ne.' ') then
                  nstation=nstation+1
                  sname(nstation)=stations(xx)
                  call which_station_type (parameter_file,
     &             sname(nstation),station_type(nstation))
                if (station_type(nstation).eq.' ') then
                  write(*,*) 'unknown type for ',sname(nstation)
                  nstation=nstation-1
                endif
                if (station_type(nstation)(1:5).eq.'IRISA') then
                   write(*,*) 'which stream (BH,LH,...) for ',
     &                 sname(nstation) 
                  read(*,'(a2)') instrument(nstation)
                elseif (station_type(nstation)(1:5).eq.'IRISB') then
                   write(*,*) 
     &             'which stream (BH,LH,00-BH,10-BH,00-LH,...) for ',
     &                 sname(nstation)
                  read(*,'(a5)') instrument(nstation)
                endif
            endif
          endif
        enddo

      else
        nstation=1
        xx=ext_stat_index
        if (stations(xx).ne.' ') then
            sname(nstation)=stations(xx)
            instrument(nstation)=ext_channel
            call which_station_type(parameter_file,
     &         sname(nstation),station_type(nstation))
        endif
      endif

      if (nstation.eq.0) then
        write(*,*) 'no stations selected'
        return 
      endif

      write(*,*)
 
      do x=1,nstation      
c
c  caclulate minimum travel time
c
        if (fixed.eqv..false.) then
          sdeqlat=deqlat
          call calc_min_tt (sname(x),c,sdeqlat,deqlon,depth,tmin,
     &                  dist(x),iout(x))

          dtmin = tmin   ! double precision

c
c  arrival time is OT plus travel time                
c
          minsec(x)= startsec + dtmin

c          write(*,*) minsec(x),startsec,dtmin
        else
          iout(x)=1
          minsec(x) = startsec
        endif

        call sectim(minsec(x),year(x),doy(x),mo(x),da(x),h(x),
     &        mi(x),sec(x))
        isec(x) = int(sec(x))

c        write(*,*) '1st arr ',sname(x),h(x),
c     &        mi(x),sec(x)

      enddo

c
c create S-file in Central database
c


c
c  make sfile name
c

      iosec = int(osec)
      if (new_sfile) then
 30   continue
        disti='R'
        call sfilname(oyear,omonth,oday,ohour,
     &      omin,iosec,cbase,disti,sfile,length )

        write(*,*) sfile

        inquire(file=sfile,exist=exist)

          if (exist) then
            if (iosec.le.58) iosec=iosec+1 
            if (iosec.eq.59) iosec=1    ! changed April 2001, lo
            goto 30
        endif
      endif

      call file_ready(sfile)

c
c  open sfile
c
      open(60,file=sfile,status='unknown')

c      write(*,*) 'writing to s-file: ',sfile
c
c write first line
c
      nordicline= ' '

      write(nordicline(2:5),'(i4)') oyear
      write(nordicline(7:8),'(i2)') omonth
      write(nordicline(9:10),'(i2)') oday
      write(nordicline(12:13),'(i2)') ohour
      write(nordicline(14:15),'(i2)') omin
      write(nordicline(17:20),'(f4.1)') osec
      write(nordicline(22:22),'(a1)') disti
      write(nordicline(24:30),'(f7.3)') deqlat
      write(nordicline(31:38),'(f8.3)') deqlon
      write(nordicline(39:43),'(f5.1)') depth
      write(nordicline(45:45),'(a1)') ' '
      write(nordicline(46:48),'(a3)') 'MAN'  
      write(nordicline(51:51),'(i1)') nstation
      write(nordicline(80:80),'(a1)') '1'

c
c  write idline
c

      call write_idline (idline,oyear,omonth,oday,ohour,omin,iosec)

      if (new_sfile) then
        write(60,'(a80)') nordicline
        write(60,'(a80)') idline      

      else
        do x=1,nhead-1
          write(60,'(a80)') data(x)
        enddo
      endif
        
      do x=1,nstation

        if (iout(x).eq.1) then

          if (station_type(x)(1:4).eq.'IRIS') then

            station_name = sname(x)
            call iris_ch
c
c  check if component will be used as trigger, channel will be used in iris_req
c
            xx=-1
            if (station_type(x)(1:5).eq.'IRISA') then
              do i=1,n_ch
                if (instrument(x)(1:2).eq.det_channels(i)(1:2)) then
                  xx=i
                endif
              enddo
            elseif (station_type(x)(1:5).eq.'IRISB') then
              do i=1,n_ch
                if (instrument(x)(1:seiclen(instrument(x))).eq.
     *            det_channels(i)(1:seiclen(instrument(x)))) then
                  xx=i
                endif
              enddo
            endif

c              if (station_n(4:4).ne.' ') then
c            cchannel = station_n(1:4) // '.' // det_channels(i)
c          else
c            cchannel = station_n(1:3) // '.' // det_channels(i)
c          endif


            channel = xx

            if (channel.gt.0) then
               call iris_req(year(x),mo(x),da(x),h(x),mi(x),sec(x),
     &         waveline,n_comp,pre_event,event_length)

              do i=1,n_comp
c
c overwrite duration
c
c                write(waveline(49:52),'(i5)') int(user_duration)
                write(60,'(a80)') waveline(i)
              enddo
            endif

          elseif (station_type(x)(1:6).eq.'SEISAN') then
            wav_text='                                          '
            if (event_length.eq.-1) then
              event_length=300
            endif
            write(wav_text,'
     &        (i4.4,i2.2,i2.2,i2.2,i2.2,i2.2,1x,i5)')
     &        year(x),mo(x),da(x),h(x),mi(x),isec(x),
     &        event_length

            call write_waveline(wave_line,'EXT',sname(x),'SEI',
     &          wav_text)
            write(60,'(a80)') wave_line

          elseif (station_type(x)(1:4).eq.'SDAS') then
            call get_components(parameter_file,sname(x),
     &         sdas_chan_list,nchan)
            wav_text='                                          '
            if (event_length.eq.-1) then
              read(sdas_chan_list(1)(62:66),'(i5)') event_length
            endif
            write(wav_text,'
     &        (i2.2,a1,i2.2,a1,i4.4,1x,i2.2,a1,i2.2,a1,i2.2,1x,i5)')
     &        da(x),'/',mo(x),'/',year(x),h(x),':',mi(x),':',isec(x),
     &        event_length

            call write_waveline(wave_line,'EXT',sname(x),'SDA',
     &          wav_text)
            write(60,'(a80)') wave_line

          elseif (station_type(x)(1:4).eq.'NAQS') then
            call naqs_channel_select(parameter_file,sname(x),
     &         naqs_chan_list,nchan)

            do i=1,nchan
              write(*,*) i,naqs_chan_list(i)
              naqs_wav_text=
     &        '                                                       '
c
c change to user defined duration
c
              if (event_length.ne.-1) then
                write(naqs_chan_list(i)(31:35),'(i5)') event_length
              endif

              write(naqs_wav_text(1:55),'(i4.4,5i2.2,1x,a35)') 
     &          year(x),mo(x),da(x),
     &          h(x),mi(x),isec(x),naqs_chan_list(i)(1:35)


              call write_waveline(wave_line,'EXT',sname(x),'NAQ',
     &          naqs_wav_text)

              write(60,'(a80)') wave_line
            enddo

          elseif (station_type(x)(1:6).eq.'AUTODR') then
            call autodrm_channel_select(sname(x),chan_list,pre_time,
     &          time_window,nchan,parameter_file)

            wav_text = '                                          '
            write(wav_text(1:20),'(i4,5i2,a5)') year(x),mo(x),da(x),
     &        h(x),mi(x),isec(x),sname(x)

            do i=1,14
              if (wav_text(i:i).eq.' ') wav_text(i:i)='0' 
            enddo

            call write_waveline(wave_line,'EVE',sname(x),'AUT',
     &          wav_text)

c
c write out as many ref lines as channels in seisnet.par, lo 16-02-2001
c
            do i=1,nchan
              if (i.gt.1) wave_line(6:8)='REF'
              write(60,'(a80)') wave_line
            enddo

c
c QNX extract
c
          elseif (station_type(x)(1:3).eq.'QNX'.or.
     &        station_type(x)(1:3).eq.'VME') then

c
c subtract preevent time
c
        if (ext_mode.eq.0) then   ! subtract user spec pre_event time
          minsec(x) = minsec(x) - float(pre_event)
        else
          minsec(x) = minsec(x) -float(pre_event)
        endif

        call sectim(minsec(x),year(x),doy(x),mo(x),da(x),h(x),
     &        mi(x),sec(x))
           isec(x) = int(sec(x))

            wav_text = '                                          '
cYY MM DD HR MN SS- (all whole numbers)
c99 08 22 01 00 00
            if (sname(x).eq.'TOV'.or.
     &          sname(x).eq.'DVV') then
              year(x)=year(x)-8
              write(*,*) ' subtracting 8 years for station: '
     &          // sname(x)
            endif
            if (year(x).ge.2000) then 
              syear = year(x) - 2000
            else
              syear = year(x) - 1900
            endif


            write(wav_text(1:23),'(5(i2,1x),i2,1x,i5)') syear,mo(x),
     &       da(x),h(x),mi(x),isec(x),event_length

            if (wav_text(1:1).eq.' ') wav_text(1:1)='0'
            if (wav_text(4:4).eq.' ') wav_text(4:4)='0'
            if (wav_text(7:7).eq.' ') wav_text(7:7)='0'
            if (wav_text(10:10).eq.' ') wav_text(10:10)='0'
            if (wav_text(13:13).eq.' ') wav_text(13:13)='0'
            if (wav_text(16:16).eq.' ') wav_text(16:16)='0'

            if (station_type(x)(1:3).eq.'QNX') then
              call write_waveline(wave_line,'EXT',sname(x),'QNX',
     &          wav_text)
            else
              call write_waveline(wave_line,'EXT',sname(x),'VME',
     &          wav_text)
            endif

            write(*,'(a80)') wave_line
            write(60,'(a80)') wave_line

          endif
            
          write(*,*)         'station             : ',sname(x)
          write(*,'(a,f10.2)')' distance in km     :',dist(x)
          write(*,'(a23,i4,4(1x,i2),1x,1f5.2)')
     &      ' minimum arrival time: ',
     &       year(x),mo(x),da(x),h(x),mi(x),sec(x)

        endif
      enddo

      if (new_sfile) then
        write(60,'(2a40)') ' STAT SP IPHASW D HRMM SECON CODA AMPLIT',
     &   ' PERI AZIMU VELO SNR AR TRES W  DIS CAZ7'
        write(60,*)

      else
        do x=nhead,nrecord
           write(60,'(a80)') data(x)
        enddo
      endif

c
c  close sfile
c
      close(60) 
c
c make waveform request if event detection not active
c
      if (wave_req_dir.ne.'NONE') then
        write(*,*) ' calling wave_request ... '
        call wave_request(sfile,wave_req_dir)
      endif

c      write(*,*) 'Return to continue'
c      read(*,*)

      write(*,*) 'sfile name: ',sfile

      return
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine which_station_type (parameter_file,name,type)
c
c return type of station, from seisnet parameter file
c
      implicit none

c station name
      character*5 name,station_name
c station type
      character*6 type
c name of SEISNET parameter file
      character*80 parameter_file
c text line
      character*80 line
c read unit  #1
       integer          read01
c logical for existing file or not
       logical          b_old
c returned code
       integer          code
c number of station in s-file
       integer number
c index
       integer ind
c number of station
       integer x

       include 'seidim.inc'
C
C    Seisan library inserts and routines...
C    ======================================
C
       include 'libsei.inc'                ! Open file definitions
C
       external sei open,                  ! Open file routine.
     &          sei close,                 ! Close file routine.
     &          sei code                   ! Error encoder.
C
C    ============= end of list ==========



            call sei open(old$+warn$,            ! Open a unknown status file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    parameter_file,        ! File name
     &                    read01 ,               ! Write unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.

      type = ' '                                                     
      x=-1
      station_name=' '
      number = -1
 10   CONTINUE
 
      read(read01,'(a)',END=20) line
c
c   find station names
c
      if (line(5:12).eq.'STATION(') then
 
         ind=index(line,')')
 
         read(line(13:ind-1),'(i3)') x
         read(line(31:35),'(a5)') station_name

         if (station_name.eq.name) number=x
      elseif (line(5:15).eq.'SYSTEMTYPE(') then

         ind=index(line,')')
 
         read(line(16:ind-1),'(i3)') x
         if (x.eq.number) then
           read(line(31:36),'(a6)') type
           goto 20
         endif
      endif
   
      goto 10
  20  continue

      write(*,*) 'station: ',name,' type: ',type
      call sei close (close$,read01,code)

      return
      end


      subroutine get_stations (parameter_file,stations,number)
c
c read station names from seisnet paramete file and also return number of stations
c

      implicit none

c station name
      character*5 stations(*)
c name of SEISNET parameter file
      character*80 parameter_file
c number of stations
      integer number
c text line
      character*80 line
c read unit  #1
       integer          read01
c logical for existing file or not
       logical          b_old
c returned code
       integer          code
c index
       integer ind
c number of station
       integer x

       include 'seidim.inc'
C
C    Seisan library inserts and routines...
C    ======================================
C
       include 'libsei.inc'                ! Open file definitions
C
       external sei open,                  ! Open file routine.
     &          sei close,                 ! Close file routine.
     &          sei code                   ! Error encoder.
C
C    ============= end of list ==========



            call sei open(old$+warn$,            ! Open a unknown status file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    parameter_file,        ! File name
     &                    read01 ,               ! Write unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.

      number = 0 
 10   CONTINUE

      read(read01,'(a)',END=20) line
c
c   find station names
c
      if (line(5:12).eq.'STATION(') then

         number = number +1

         ind=index(line,')')

c         read(line(13:ind-1),'(i3)') x
         read(line(31:35),'(a5)') stations(number)

      endif
  
      goto 10
  20  continue

c      write(*,*) 'station: ',name,'type: ',type

      call sei close (close$,read01,code)

      return
      end


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine add_time(text,i)
      implicit none
      character*20 text,out
      integer i
      integer year,month,day,hour,min,sec,doy
      real rsec
      double precision msec

      out = ' '

      read(text(1:4),'(i4)') year
      read(text(6:7),'(i2)') month
      read(text(9:10),'(i2)') day
      read(text(12:13),'(i2)') hour
      read(text(15:16),'(i2)') min
      read(text(18:19),'(i2)') sec
      rsec=sec

      call timsec(year,month,day,hour,min,rsec,msec)
      msec=msec+float(i)
      call sectim(msec,year,doy,month,day,hour,min,rsec)
      sec=int(rsec)
      write(out,'(i4.4,a1,i2.2,a1,i2.2,a1,i2.2,a1,i2.2,a1,i2.2)') 
     &   year,'/',month,'/',day,'/',
     &   hour,':',min,':',sec

      do i=1,19
        if (out(i:i).eq.' ') out(i:i)='0'
      enddo
      write(*,*) ' end-time ' // out // ' #'

      return
      end



      subroutine systim(t)
      implicit none
c
c return system time as yyyymmddhhmmss
c
      character*14 t
      integer time(3),date(3),i
      logical sun,pc,linux
      call computer_type(sun,pc,linux)
c
c   get system time on sun
c
       if(sun.or.linux) then
         call itime(time)
         call idate(date)
         write(t,'(i4,5i2)')
     *   date(3),date(2),date(1),time(1),time(2),time(3)
       endif
c
c   check for blanks
c
       do i=1,14
          if(t(i:i).eq.' ') t(i:i)='0'
       enddo

      return
      end




      subroutine gmttim(ctime)
c routine to give gmttime as 14 chars

      implicit none
      integer*4 stime, ta(9)
      integer time,i
      character*14 ctime

      stime = time ()
      call gmtime(stime,ta)

c month is 0-11
      ta(5) = ta(5) + 1

c add century
      if (ta(6).lt.50) then
        ta(6) = ta(6) + 2000
      else
        ta(6) = ta(6) + 1900
      endif

      write(ctime,'(i4,5i2)') ta(6),ta(5),ta(4),ta(3),ta(2),ta(1)

c
c   check for blanks
c
       do i=1,14
          if(ctime(i:i).eq.' ') ctime(i:i)='0'
       enddo
      return
      end




      subroutine GETREFID(it,refid,reflin)
      implicit none
      integer it, trimbeg,trimlen, i, j, jj
      character refid*30, line*80, reflin*80
      integer seiclen
c
      rewind(it)
      refid='(No_REF_ID_received)'
      i=0
c
   1  read(it,'(a)',end=999,err=999) line
      if(line.ne.' ') goto 1                  ! read until end of mail-header
c
   2  read(it,'(a)',end=999,err=999) line
      i=i+1
      call CASEFOLD(line)
      if(line(1:6).eq.'REF_ID')then
         reflin=line
         line(1:6)=' '
         refid=line(trimbeg(line):trimlen(line))
c
c cut off reg agency if there
c
         jj=-1
         do j=1,seiclen(refid)
           if (refid(j:j).eq.' '.and.jj.eq.-1) then
             jj=j - 1
           endif
         enddo
         if (jj.ne.-1) refid=refid(1:jj)

         RETURN
      endif
      if(i.gt.50) goto 999 ! search only in the first 5-6 lines for MSG_ID

      goto 2
c
 999  continue
      reflin=' '
      rewind(it)
c
      RETURN
      end ! of subr. getmsgid

      subroutine GETMSGID(it,msgid,reflin)
      implicit none
      integer it, trimbeg,trimlen, i
      character msgid*30, line*80, reflin*80
c
      rewind(it)
      msgid='(No_MSG_ID_received)'
      i=0
c
   1  read(it,'(a)',end=999,err=999) line
c2000      if(line.ne.' ') goto 1              ! read until end of mail-header
      call CASEFOLD(line)
      if(line(1:5).ne.'BEGIN') goto 1          ! read until start of message
c
   2  read(it,'(a)',end=999,err=999) line
      i=i+1
      call CASEFOLD(line)
      if(line(1:6).eq.'MSG_ID')then
         reflin=line
         line(1:6)=' '
         msgid=line(trimbeg(line):trimlen(line))
         RETURN
      endif
      if(i.gt.5) goto 999 ! search only in the first 5-6 lines for MSG_ID
      goto 2
c
 999  continue
      reflin=' '
      rewind(it)
c
      RETURN
      end ! of subr. getmsgid



      subroutine freeunit(iunit)
c
c     get a free fortran file-unit number that is not yet in use
c
      implicit none
c
c     Purpose: Get a free FORTRAN-unit
c
c     Output: iunit  a free (file-)unit number
c
c     Urs Kradolfer, 16.7.90
c
      integer iunit
      logical lopen
c
      do iunit=10,999
         if(iunit.eq.999) stop'FREEUNIT>>> no free unit found!'
         inquire(unit=iunit,opened=lopen)
         if(.not.lopen) RETURN
      enddo
c
      RETURN
c
      end

      subroutine filecheck(it,startnotfound,gseformat,beglin)
c
c     check, if the incoming file is an AutoDRM request mail
c
      implicit none
      integer it, i
      character line*80, beglin*80, gseformat*10
      logical startnotfound
c
c     First, skip the header of the mail:
c
   1  read(it,'(a)',end=999) line
      if(line.ne.' ') goto 1
c
c     Now check, if a line is starting with BEGIN or BEGIN GSE2.0 :
c
      i=0
   2  read(it,'(a)',end=999) line
      if(line.eq.' ') goto 2  ! do not count blank lines
      if(line(1:5).eq.'-----') goto 2  ! lines: --- body part 1 ---   etc.
      i=i+1
      call CASEFOLD(line)
      if(line(1:5).eq.'BEGIN')then
         beglin=line
         if(INDEX(line,'GSE2.0').gt.0) gseformat='GSE2.0'
         RETURN
      endif
      if(i.gt.3) goto 999 ! search only for first 2-3 lines in a mail
      goto 2
c
 999  startnotfound=.true.
      beglin=' '
      RETURN
      end ! of subr. filecheck


      integer function TRIMLEN(t)   ! Urs Kradolfer, June 1986
c
c     Call:    nc=TRIMLEN(char)
c
c          --> nc says, how many characters the input-string has
c              (ignoring trailing blanks!).
c
      implicit none
      character t*(*)
      do 1 trimlen=LEN(t),1,-1
    1    if(t(trimlen:trimlen).ne.' ')RETURN
      trimlen=1
      end ! of integer function trimlen

      integer function TRIMBEG(t)   ! Urs Kradolfer, 17.1.90
c
c     Call:    nc=TRIMBEG(char)
c
c          --> nc returns the position of the first characters of
c              the input-string (ignoring leading blanks!).
c
      implicit none
      character t*(*)
      do 1 trimbeg=1,LEN(t)
    1    if(t(trimbeg:trimbeg).ne.' ')RETURN
      trimbeg=1
      end ! of integer function trimbeg

      subroutine casefold(cn) ! Urs Kradolfer, 20. Feb. 1990
c
c     put all characters of the inout string into UPPER CASE letters
c
c
c Input : character-string of any length
c Output: same character-string, but all letters are CAPITAL now
c         (other characters are not changed)
c
c   --> this subr. needs the function TRIMLEN !
c
c Call  : call CASEFOLD(charstring)
c
      implicit none
      character cn*(*)
      integer i, trimlen,ilen, ival
      ilen=trimlen(cn)
      do i=1,ilen
         ival=ICHAR(cn(i:i))
         if(ival.ge.97.and.ival.le.122)then
            cn(i:i)=CHAR(ival-32)
         endif
      enddo

      RETURN
      end ! of subr. casefold

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine xxx_make_event_list(station,dayfile)
c
      implicit none
      character*5 station
c filenames
      character*80 infile,outfile,dayfile
c text line
      character*80 line
c time
      character*8 time
      integer hr,min,sec,total
c original time       
      integer shr,smin
c date
      character*8 date
c component
      character*5 comp
c counter
      integer i
c switch day
      logical midnight 
c index, counter
      integer ind,x
c daystring    example: 97 03 04 09:19:46
      character*17 daystring,prev_daystring
      integer seiclen
      

   5  CONTINUE
 
c
c open files
c
   
      ind=seiclen(station)
      if (ind.lt.0) then
        ind=4
      endif

      infile=station(1:ind) // '.LOG'
      outfile=station(1:ind) // '.EVE'

      open(1,file=infile,status='old')
      open(2,file=outfile,status='unknown')
      open(3,file=dayfile,status='unknown')
      
c
c read date of DAY file, will be used if no new detections since last transfer
c
      read(3,'(a17)') prev_daystring
      close(3) 
      open(3,file=dayfile,status='unknown')

      do  i=1,13,1
        read(1,*)
      enddo

      time=' '
      date=' '

  10  CONTINUE
      midnight=.FALSE.
      read(1,'(a80)',end=100) line
      read(line(67:72),'(a5)') comp
      read(line(28:35),'(a8)') time
      if (station(1:3).eq.'KBS') then
        if (comp(1:3).eq.'LHZ') comp='Z LP '
        if (comp(1:3).eq.'LHE') comp='E LP '
        if (comp(1:3).eq.'LHN') comp='N LP ' 
        if (comp(1:3).eq.'BHZ') comp='Z VBB'
        if (comp(1:3).eq.'BHE') comp='E VBB'
        if (comp(1:3).eq.'BHN') comp='N VBB'
      endif
      if (comp(3:5).eq.'VBB'.OR.
     &    comp(3:4).eq.'LP')   THEN
        read(time(1:2),'(i2)') hr
        read(time(4:5),'(i2)') min
        read(time(7:8),'(i2)') sec
        shr=hr
        smin=min
        total=hr*3600+min*60+sec       
        if (comp(3:3).eq.'L') then
          total=total-1800
        else
          total=total-60
        endif
        if (total.LT.0) THEN
          total=86340
          midnight=.TRUE.
        endif
        hr=abs(total/3600)
        min=abs((total-hr*3600)/60)
        sec=total-hr*3600-min*60
        write(time(1:2),'(i2)') hr
        write(time(4:5),'(i2)') min
        write(time(7:8),'(i2)') sec
        read(line(15:16),'(a2)') date(1:2)
        read(line(22:26),'(a5)') date(4:8)
c
c only trigger on Z-component is used, but all three comp. will be transferred
c

c        if (station.eq.'KBS'.or.
c     &    (station.eq.'KONO'.and.comp(1:4).eq.'Z LP')) then

         if (comp(1:4).eq.'Z LP'.or.comp(1:5).eq.'Z VBB') then
          write(2,*) comp
          write(2,*) date,' ',time
          
          comp(1:1)='E'
          write(2,*) comp
          write(2,*) date,' ',time
          
          comp(1:1)='N'
          write(2,*) comp
          write(2,*) date,' ',time

c        else
c          write(2,*) comp
c          write(2,*) date,' ',time
        endif
      endif
        
      if (comp(3:5).eq.'VBB'.OR.
     *    comp(3:4).eq.'LP') THEN
       
      endif
      

      GOTO 10

 100  CONTINUE
      write(2,*)

c write day file
c 97 03 04 09:19:46
      do x=1,17
        daystring(x:x)=' '
      enddo

      write(daystring(1:2),'(a2)') date(1:2) 
      write(daystring(4:5),'(a2)') date(4:5)
      write(daystring(7:8),'(a2)') date(7:8)
        smin=smin+1
        sec=0
        if (smin.eq.60) then
          smin=0
          shr=shr+1
        endif
        if (shr.eq.25) hr=1
      write(daystring(10:11),'(i2)') shr
      daystring(12:12)=':'
      write(daystring(13:14),'(i2)') smin
      daystring(15:15)=':'
      write(daystring(16:17),'(i2)') sec
 
      if (daystring(1:2).eq.'  ') then      ! no event since last transfer
         write(3,'(a)') prev_daystring
      else
        write(3,'(a)') daystring
      endif

 200  CONTINUE
      close(1)
      close(2)
      close(3)
     
      RETURN
      END



      subroutine xxx_dirf (eventfile) 
c
c subroutine: dirf - makes filenr.lis
c
      implicit none

      character*20 eventfile     ! name of *.EVE file
      character*70 filename      ! filename of waveform file 
      character*80 text          ! output text

      integer c,x                ! counter

      open(1,file=eventfile,status='unknown')
      open(2,file='filenr.lis',status='unknown')
 
      x=0

 300  continue

      read(1,'(a)',end=400) filename
      x=x+1
      do c=1,80
        text(c:c)=' '
      enddo 
      text=' #     ' // filename
      write(text(3:5),'(i3)') x
      if (filename(5:5).eq.'_'.or.filename(5:5).eq.'-') then
        write(2,'(a80)') text
      endif

      goto 300

 400  continue


      close(1)
      write(2,*)
      close(2)

      return
      end



      subroutine xxx_last_event(agency,type)
c
c find last s-file in database and give time as output
c this subroutine is not used
c

      implicit none

c agency
      character*3 agency
c station type
      character*6 type
c seisan topdirectory
      character*60 top_directory
c seisan readings directory
      character*70 rea_dir
c event directory
      character*80 event_dir
c work file
      character*80 wor_file
c date and year of database
      character*6 time_dir
c dirctory delimiter character
      character*1 dchar
c index
      integer ind
c date and time
      integer time(3),date(3)
c counter
      integer i
c command for ls
      character*300 text
c text line
      character*80 line
c filename
      character*17 filename
c time output if type is iris
      character*17 iristime

c time read from event filename
      integer etime(3),edate(3)
c exists file in event directory
      logical file_exist,notagain
      integer seiclen

c
c get directory structure
c

      call topdir(top_directory)

      ind=seiclen(top_directory)

      call dir_char(dchar)         ! dirctory delimiter character


c
c readings directory
c
      rea_dir = top_directory(:ind)//
     &           dchar // 'REA' // dchar // agency // dchar

      wor_file = top_directory(:ind)//
     &           dchar // 'WOR' // dchar // 'onpro.tmp'
c
c get date and time
c

      call itime(time)
      call idate(date)

c      if (date(3).lt.2000) then
c        date(3)=date(3)-1900
c      else
c        date(3)=date(3)-2000
c      endif

      file_exist=.FALSE.
      notagain=.FALSE.

 500  continue

c
c get listing of event directory
c

      write(time_dir,'(i4,a1,i2,a1)') date(3),dchar,date(2),dchar

      do i=1,8
         if (time_dir(i:i).eq.' ') time_dir(i:i)='0'
      enddo

      ind=seiclen(rea_dir)
      event_dir=rea_dir(:ind) // time_dir

      ind=seiclen(event_dir)

      text=' '
      text = 'ls -1 ' // event_dir(:ind) //' > '// wor_file // '  '

      ind=index(text,'  ') -1
      call systemc(text,ind)

c
c read wor_file and get time and date of last eveent file
c

      ind=index(wor_file,' ')
      call file_ready(wor_file)
      open(20,file=wor_file,status='old')

 600  continue

      read(20,'(a)',end=700) line

      if (line(3:3).eq.'-'.and.line(8:8).eq.'-') then
        read(line,'(a17)') filename
        file_exist=.TRUE.
      endif


      goto 600

 700  continue

c
c if no event file exists look in month before, if still no file
c time is set to the first of last month
c
      if (file_exist.eqv..FALSE..and.notagain.eqv..FALSE.) then
        notagain=.TRUE.
        if (date(2).ge.2) then
          date(2)=date(2)-1
          goto 500
        elseif (date(2).eq.1) then
          date(2)=12
          date(3)=date(3)-1
          goto 500
        endif
      elseif (file_exist.eqv..FALSE..and.notagain.eqv..TRUE.) then
c write 1st of last month
        write(filename(1:2),'(a2)') '01'       ! day
        write(filename(16:17),'(i2)') date(2)  ! month
        write(filename(14:15),'(i2)') date(3)  ! year
        filename(4:10)='0000-00'
      endif
c
c read time from event filename
c
      read(filename(1:2),'(i2)') edate(1)    ! day
      read(filename(16:17),'(i2)') edate(2)  ! month
      read(filename(14:15),'(i2)') edate(3)  ! year
      read(filename(4:5),'(i2)') etime(1)    ! hour
      read(filename(6:7),'(i2)') etime(2)    ! minute
      read(filename(9:10),'(i2)') etime(3)   ! seconds

c
c write iris time
c

      do i=1,17
         iristime(i:i)=' '
      enddo
      write(iristime(1:2),'(i2)') edate(3)
      write(iristime(4:5),'(i2)') edate(2)
      write(iristime(7:8),'(i2)') edate(1)
      write(iristime(10:11),'(i2)') etime(1)
      write(iristime(13:14),'(i2)') etime(2)
      write(iristime(16:17),'(i2)') etime(3)

      if (type.eq.'IRIS') then
        write(*,*) iristime
      endif

 800  continue

      close(20)
      return

      end


      subroutine copy_det_file(source,event_dir,wave_req_dir,option)
      implicit none
      integer seiclen,option
      character*(*) source,event_dir,wave_req_dir
      character*240 text

c
c copy s-file to event detection directory if event detection active
c
      if (option.ge.10.and.event_dir.ne.'NONE') then
        text='cp '//source(1:seiclen(source))//' '//
     &        event_dir(1:seiclen(event_dir))
        write(*,*) text(1:seiclen(text))
        call systemc(text,seiclen(text))
      endif
c
c make waveform request if event detection not active
c
      if (option.eq.1.and.wave_req_dir.ne.'NONE') then
        call wave_request(source,wave_req_dir)
      endif

      return
      end



ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c Subroutine to remove '0' from time string
c
c
c      subroutine ctime(intim)
c
c      implicit none
c
c      character*2 intim
c      integer intout
c
c      read(intim,'(i2)') intout
c      write(*,*) intout
c
c      return
c      end
c
c   end of subroutine ctime
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine get_components(pfile,station_name,chan_list,nchan)
c
c read component codes from seisnet parameter file
c
      implicit none

      character*(*) station_name
      character*5 station
      character*(*) chan_list(*)
      integer nchan
      character*(*) pfile

      include 'seidim.inc'
      include 'libsei.inc'                ! Open file definitions
      external sei open,                  ! Open file routine.
     &         sei close,                 ! Close file routine.
     &         sei code                   ! Error encoder.
      integer read01,code
      logical b_old
      integer i,j,x,ind
      character*80 par_line

      nchan=0
      x=-1

      call file_ready(pfile)

      call sei open(old$+warn$,                  ! Open a unknown status file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    pfile,                 ! File name
     &                    read01 ,               ! Write unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)

10    continue
      read(read01,'(a)',END=15) par_line

c
c find station in file
c
      if (par_line(5:12).eq.'STATION(') then
         ind=index(par_line,')')
         read(par_line(31:35),'(a5)') station
         if (station.eq.station_name) then
           read(par_line(13:ind-1),'(i3)') x
         endif
c
c  find components for station name
c
      elseif (par_line(5:14).eq.'COMPONENTS') then

        ind=index(par_line,')')

        read(par_line(16:ind-1),'(i3)') i

        if (i.eq.x) then
          nchan=nchan+1
cet COMPONENTS(1)             "*.*                 12345     12345"
          read(par_line(32:66),'(a)') chan_list(nchan)
        endif
      endif

      goto 10
15    continue
c
c close parameter file
c
      call sei close (close$,read01,code)

c
c  output
c
      if (x.eq.-1) then
         write(*,*) 'station not in list !!!'
         return
      else
        write(*,*) station_name,' found ...'
        do x=1,nchan
          write(*,*) chan_list(x)
        enddo
      endif

      return
      end


