
c    Copyright 2003 Lars Ottemoller and Jens Havskov
c
c    This file is part of SEISNET.
c
c    SEISNET is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    Foobar is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with Foobar; if not, write to the Free Software
c    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USa

c
c This file contains SEISNET FORTRAN subroutines that are used for QNX Seislog
c type stations.
c

c
c List of subroutine in alphabetical order:
c
c   vmeseislog_cnordic - convert lgs output to Nordic file format
c   update_vmeseislog - update s-file after waveform file transfer

c
c Changes:
c
c  04 Dec 2001 lo: first version
c

      subroutine vmeseislog_cnordic(station_name,base,lastevent,
     &     event_dir,wave_req_dir,det_wav_option)
c
c convert VME Seislog detection list (lgs) to SEISAN Nordic format
c
c Lars Ottemoller, Dec 2001 
c

      implicit none

      include 'seidim.inc'
      integer  seiclen      ! character length

c -- station name
      character*5 base
c -- time
      integer hour, minute,sec
      real seconds 
c -- date
      integer year,month,day
c -- date and time of last event
      integer last_hour, last_minute,last_sec
      integer last_year,last_month,last_day
c -- linetype in Nordic format
c    distance indicator
      character*1 linetype, dist
c -- reporting agency
      character*3 agency
c -- line of QUAKE and nordic file resp.
      character*80 nordicline,phaseline,idline
      character*90 line,o_line      ! line with more than 80 chars
c -- top-directory
       character*60 top_directory
c -- index
       integer ind,inds
c -- directory separator character
      character*1 dchar
c -- text strings
       character*80 chr_text_out
c -- number of events
       integer counter
c -- instrument type in Nordic format
       character*1 instout
c -- idnumber
       character*14 idnumber,idtime
c -- counter
       integer i
c -- waveform file line
       character*80 waveline
c time (day,hour,min)
       character*6 time
c length of line
       integer l
c coda length - duration
       integer coda
c  name of S-file
       character*80 sfile
c station name
       character*5 station_name
c number of events
       integer nevent
c file exists
       logical exist
c wav text
       character*42 wav_text
c flag
       logical flag

       character*40 lastevent ! last event in data base
       character*40 currentevent ! current event in data base
       integer      stat     ! flag to compare date of events
       integer  length       ! length of sfile nam,e
c option for waveform transfer
       integer det_wav_option
c directories for network event detection and waveform transfer
       character*(*) event_dir,wave_req_dir
       character*14 last_detection

      nevent=0
      last_detection = ' '

c
c get directory structure
c

      call topdir(top_directory)

      ind=seiclen(top_directory)
      inds=seiclen(station_name)
      if (inds.eq.-1) inds=4

      call dir_char(dchar)         ! dirctory delimiter character

c
c open files
c
      call file_ready(station_name(1:inds)//'.INP')
      open(50,file=station_name(1:inds)//'.INP',status='old')
      ind=seiclen(chr_text_out)

c
c set default parameters
c
      agency=station_name(1:3) 
      linetype= '1'

      counter=0

c
c get date and time of last event
c
      read(lastevent(1:4),'(i4)') last_year
      read(lastevent(5:6),'(i2)') last_month
      read(lastevent(7:8),'(i2)') last_day
      read(lastevent(9:10),'(i2)') last_hour
      read(lastevent(11:12),'(i2)') last_minute
      read(lastevent(13:14),'(i2)') last_sec
      last_detection=lastevent
      do i=1,14
        if (last_detection(i:i).eq.' ') last_detection(i:i)='0'
      enddo

  10  continue
c
c read original line 
c
      read(50,'(a)',end=900) o_line
      line=o_line

c xxx      if (o_line(73:80).eq.'TOV_13_X') goto 20

c
c  example of line
c825  2  32  2  1  9:24:33.60    52    19      14103834 M 0202_01_0924_33T.BCV_1
c44  2 172  6 21 10: 1:33.45   150    -1      -1 88890 M 0206_21_1001_33T.BCV_18
c330 93 323 11 19 17: 0:34.30 121      -1      -1   953 9311_19_1700_34Y.DVV_07_X
c 47  2 172  6 21 10:57:33.39   240    -1      -1 90570  0206_21_1057_33T.BCV_18 
c06 94 165  6 14  4:59:59.19  61      -1      -1  5809 9406_14_0459_58T.TOV_13_X
c                            

c 
c in case different lgs output
c
      if (line(58:58).eq.' ') then
        line(57:seiclen(o_line)-2)=o_line(59:seiclen(o_line))
      elseif (line(57:57).eq.' ') then
        line=' '//o_line(1:55)//o_line(58:seiclen(o_line))
      elseif (line(55:55).eq.' ') then
        line=' '//o_line(1:seiclen(o_line))
      endif

      if (line(61:61).ne.'_'.and.line(64:64).ne.'_') THEN
        GOTO 20 
      endif

      counter=counter+1

      wav_text = '                                          '
c
c put log dpx number
c

      wav_text(1:4) = line(1:4)


      call write_waveline(waveline,'EVE',station_name,'VME',
     &     wav_text)


c200 97 252  9  9  0:54:29.35
c330 93 323 11 19 17: 0:34.30 
      read(line(6:7),'(i2)') year
      if (year.gt.5) year=year+8
c
c  add century
c
      if (year.gt.50) then
         year = year + 1900
      else
         year = year + 2000
      endif
      read(line(13:14),'(i2)') month
      read(line(16:17),'(i2)') day

      read(line(19:20),'(i2)') hour
      read(line(22:23),'(i2)') minute
      read(line(25:29),'(f5.2)') seconds

c  read signal duration
      read(line(30:33),'(i4)') coda
 
      sec= abs(seconds)

      write(currentevent,'(i4,5(i2))') year,month,day,hour,
     *minute,sec
      do i=1,14
         if (currentevent(i:i).eq.' ') then
           currentevent(i:i)= '0'
         endif
      enddo

c
c decide to convert or not
c if date of current is newer (greater) than date of last event
c then convert it, otherwise, bypass
c
c      call compare(currentevent,lastevent,stat)
c      if (stat.le.0) goto 25
c lot 03052002 changed since lgs output may not be in cronological order, 
c              only skip if file already exists 
c

  15  continue

      last_detection=currentevent

      instout='S'
      dist= 'R'
      
c
c write nordic line
c
      nordicline= ' '

      write(nordicline(2:5),'(i4)') year
      write(nordicline(7:8),'(i2)') month
      write(nordicline(9:10),'(i2)') day   
      write(nordicline(12:13),'(i2)') hour
      write(nordicline(14:15),'(i2)') minute
      write(nordicline(17:18),'(i2)') sec
      write(nordicline(22:22),'(a1)') dist
c      write(nordicline(45:45),'(a1)') '*'
      write(nordicline(46:48),'(a3)') agency
      write(nordicline(51:51),'(a1)') '1'
      write(nordicline(80:80),'(a1)') linetype

c
c  write idline
c
      call write_idline(idline,year,month,day,hour,minute,sec)

      phaseline=' '
      write(phaseline(2:6),'(a5)') station_name
      write(phaseline(7:7),'(a1)') instout
      write(phaseline(8:8),'(a1)') 'Z'
      write(phaseline(10:11),'(a2)') 'EP'
      write(phaseline(16:16),'(a1)') 'A' 
      write(phaseline(19:20),'(i2)') hour
      write(phaseline(21:22),'(i2)') minute
      write(phaseline(24:28),'(f5.2)') seconds
      write(phaseline(30:33),'(i4)') coda

c
c open output file
c
      call sfilname(year,month,day,hour,
     &      minute,sec,base,dist,sfile,length )

c
c  skip if file exists
c
      inquire(file=sfile,exist=exist)

      if (exist) then
        write(*,*) ' old detection: '//sfile(1:seiclen(sfile))
        goto 20
      endif

      call file_ready(sfile)

      open(60,file=sfile,status='new')
c
c output
c
      write(60,'(a80)') nordicline
      write(60,'(a80)') idline
      write(60,'(a80)') waveline
      write(60,'(a80)') ' STAT SP IPHASW D HRMM SECON CODA AMPLIT'
     & //' PERI AZIMU VELO SNR AR TRES W  DIS CAZ7'
c      write(60,'(a80)') phaseline
      write(60,'(a)') ' '

      close(60)
 
      call copy_det_file(sfile,event_dir,wave_req_dir,det_wav_option)

      nevent=nevent + 1

  20  CONTINUE
      goto 10

 900  continue

      close(50)
      close(60)

      write(*,*) '... in total ',nevent,' detections ...'
      write(*,*) '... last detection ',last_detection,' ...'
  
      return
      end    ! end of qnx_cnordic


      subroutine update_vmeseislog(S_file,wave_vme,wave_name,autopick)
c
c update S-file after requested waveform file has been transferred
c
c Lars Ottemoller, Dec 2001
c
      implicit none

      include 'libsei.inc'                ! Open file definitions
      include 'seidim.inc'

      character*255 S_file              ! name of S-file
      character*40  wave_name,wave_vme  ! name of waveform file
      character*8 autopick              ! name of autopick routine

      logical      there                ! test if file exists
 
c-- event data
       character*80     data(max_data)
c-- exp indicator
       character*1      exp
c-- event type
       character*1      type
c-- number of recors for event
       integer          nrecord
c-- number of header lines
       integer          nhead
c number of phases 
       integer nphase
c read unit
       integer read01
c number of stations
       integer nstat
c-- id line number
       integer id
c counter
      integer x,i1,i2
c line with 'REQUEST'
      integer requ_line
c waveform line
      character*80 waveline
c autopick line
      character*80 autopick_line
c index
      integer ind
c flag if autopick line
      logical flag
      integer seiclen

c
c   check if file exists
c
      inquire (file=S_file,exist=there)
      if (.NOT.THERE) then
         write(*,*) S_file,' does not exist !'
         return
      endif
 

c
c  open S-file
c
      call file_ready(S_file)
      open(unit=8,file=S_file,status='old')

c
c read event file
c
      read01=8
      call indata(read01,nstat,nphase,nhead,nrecord,type,exp,data,id)
      close(8)

      if (data(nrecord).eq.' '.and.data(nrecord-1).eq.' ') then
        nrecord=nrecord-1
        nhead=nhead-1
      endif

c
c look for "REQ"
c
      do x=1,nhead
        if (data(x)(76:78).eq.'REQ') then
          read(data(x)(23:27),'(i5)') i1
          read(wave_vme(1:5),'(i5)') i2
          if (i1.eq.i2) then 
            requ_line=x
          endif
        endif
      enddo

c
c  make autopick_line, use write_autopick_line new April 2001, lo
c

      call write_autopick_line(data(requ_line),autopick_line,
     &    wave_name,autopick,flag)

c
c update line
c
        data(requ_line)(76:78) = 'TRA'   ! changed April 2001, lo

c
c blank new line
c
        do x=1,80
           waveline(x:x) = ' '
        enddo
c
c write info to new line
c
        write(waveline(2:41),'(a40)') wave_name
        write(waveline(80:80),'(a1)') '6'
c
c update according to status of file transfer
c
        call file_ready(S_file)
        open(8,file=S_file,status='unknown')

        do x=1,nhead-1
           write(8,'(a80)') data(x)
        enddo

        if (flag) write(8,'(a80)') autopick_line
      
        write(8,'(a80)') waveline

        do x=nhead,nrecord
          write(8,'(a80)') data(x)
        enddo
c
c close updated S-file and end process
c

        close(8)


      return
      end



