
c    Copyright 2003 Lars Ottemoller and Jens Havskov
c
c    This file is part of SEISNET.
c
c    SEISNET is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    Foobar is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with Foobar; if not, write to the Free Software
c    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USa

c program that starts different autoprocessing routines


      program autopro 

c changes:
c
c  16 Jan 2000 - put in to send out email
c  08 Feb 2000 - use file_ready
c  
c

      implicit none

      include 'seidim.inc'
 
C
C    Seisan library inserts and routines...
C    ======================================
C
       include 'libsei.inc'                ! Open file definitions
C
       external sei open,                  ! Open file routine.
     &          sei close,                 ! Close file routine.
     &          sei code                   ! Error encoder.
C
C    ============= end of list ==========
C

c-- event data
       character*80     data(max_data)
c-- exp indicator
       character*1      exp
c-- event type
       character*1      type
c-- number of recors for event
       integer          nrecord
c-- number of header lines
       integer          nhead
c-- number of stations
       integer          nstat
c -- event id
       integer id
c-- number of phases
       integer nphase
c-- start and end time of select
       character*14    start_time,end_time,time_now
c-- data base name if not agency (blank)
       character*40     base_name
c-- event file name
       character*80     evfile
c-- select key
       character*10     key
c-- see subroutine:find...
       integer          status,new_month,fstart,event_no
c-- true if file exists (CLK)
       logical exist
c logical for existing file or not
       logical          b_old
c returned code
       integer          code
c read unit #1
      integer          read01,write01,write02
      integer nars                            ! number of arguments
      character*320  args(8)                  ! arguments
      integer year,month,day,hour,min,doy     ! time and date
      double precision msec                   ! time in second
      real rsec                               ! real sec
      real time_diff                          ! days back in seconds
      integer time_back                       ! days back
      integer x,y                             ! counter
      logical new                             ! is event new?
      logical picked                          ! is file picked
      character*80 message                    ! message to hyp 
      logical auto_locate, autopick           ! indicating automatic processing
      logical email_alert                     ! true if send out emails desired
      character*320 email_address             ! address for email alert
      character*400 line
      real min_mag,mag
      character*14 time
      integer seiclen
      integer alert_file

c
c version
c
      call print_ver 
     & ('SEISNET - autopro - version: 2.0.5   date: 06/04/2000')

c 
c open list of alerts file
c
      call sei open(unknown$,                    ! Open an old file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    'alert.lis',           ! File name
     &                    alert_file,            ! Read unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.

c
c  get arguments
c 
       call get_arguments(nars,args)
c
c init
c
      autopick = .false.
      email_alert =.false.

      if (nars.eq.7) then

        base_name(1:5) = args(1)(1:5) 
        do x=2,5
          if (base_name(x:x).eq.' ') base_name(x:x)='_'
        enddo
        time_now=args(2)
        end_time=time_now
        read(args(3),'(i3)') time_back

        if (args(4).eq.'YES') auto_locate=.TRUE.
        if (args(5).ne.'NO') then
          autopick  =.TRUE.
        endif
        if (args(6).ne.'NO') then
          email_alert = .TRUE.
          read(args(6),'(f3.1)') min_mag
        endif
        if (args(7).ne.'NO') then
          email_address = args(7)
        else
          email_alert = .FALSE.
        endif

      ELSE
        write(*,*) 'wrong number of parameters !!! '
        stop
      ENDIF

      read(time_now,'(i4,2i2)') year,month,day
      hour=12
      min=0
      rsec=0
      
      call timsec(year,month,day,hour,min,rsec,msec)
      time_diff = time_back * 86400.0
      msec = msec - time_diff
      call sectim(msec,year,doy,month,day,hour,min,rsec)

      write(start_time,'(i4,2i2)') year,month,day

      do x=1,8
        if (start_time(x:x).eq.' ') start_time(x:x)='0'
        if (end_time(x:x).eq.' ') end_time(x:x)='0'
      enddo 

      write(*,*) 'processing .... '
      write(*,*) 'base       : ',base_name(1:5)
      write(*,*) 'start time : ',start_time
      write(*,*) 'end time   : ',end_time
      write(*,*) 
 
c
c  start read and write loop
c

 5     continue
c-- always use next event
       key='          '
       CALL findevin
     * (base_name,start_time,end_time,key,0,
     * event_no,evfile,fstart,new_month,status)
       if(status.eq.0) then
c--   (CLK)
         inquire(file=evfile,exist=exist)
c--   (CLK)

         call put_env_event(evfile)

         if (exist) then
            call file_ready(evfile)
            call sei open(old$+warn$,            ! Open an old file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    evfile,                ! File name
     &                    read01,                ! Read unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.

c
c read s-file
c
         call indata(read01,nstat,nphase,nhead,nrecord,type,exp,data,id)


c
c  close file
c
           call sei close (close$,read01,code)
 
c
c  find out if new event, means not processed before
c
         x=1
         new=.false.
         picked = .false.

  10     continue

         if (data(x)(80:80).eq.'I') then
 
           if (data(x)(9:11).eq.'NEW') new=.true.
           
           goto 20
         endif

 
         if (x.eq.nhead) goto 20

         x=x+1
         goto 10

  20     continue
         

c
c  print event name
c

         if (autopick) then


           do x=1,nhead
c
c start autopic for each waveform file
c
cNET WAV EVE KONO1IRI 2000-01-05-1107-18S.KONO__001_B10E         FIND          3
              if (data(x)(80:80).eq.'3'.and.
     &            (data(x)(66:72).eq.'AUTOPIC'.or.
     &            data(x)(76:78).eq.'API')) then

                 message = 'autopic ' // data(x)(23:64)
                 call put_message('autopic',message)

c                 call system('autopic')
                 call systemc('autopic',7)

                 picked =.TRUE.

c
c start findp for each waveform file
c
              elseif (data(x)(80:80).eq.'3'.and.
     &              (data(x)(66:70).eq.'FINDP'.or.
     &               data(x)(76:78).eq.'FIP')) then

                 if (data(x)(66:70).eq.'FINDP') then
                   message = 'findp ' // data(x)(23:64)
                 else
                   message = 'findp ' // data(x)(23:74)
                 endif
                 call put_message('findp',message)

                 call systemc('findp',5)

                 picked =.TRUE.

              elseif (data(x)(80:80).eq.'3'.and.   ! added April 2001, lo
     &              data(x)(76:78).eq.'ASI') then

                 message = 'autosig ' // data(x)(23:74)
                 call put_message('autosig',message)
                 call systemc('autosig',7)
                 picked =.TRUE.
              endif
           enddo
         endif

c
c update sfile
c

           if (picked) then

   

              call file_ready(evfile)
              call sei open(old$+warn$,            ! Open an old file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    evfile,                ! File name
     &                    read01,                ! Read unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.
 
         call indata(read01,nstat,nphase,nhead,nrecord,type,exp,data,id)

                call sei close (close$,read01,code)
   
                call file_ready(evfile)
                call sei open(old$+warn$,            ! Open an old file.
     &                      ' ',                   ! Prompt file name (n/a).
     &                      evfile,                ! File name
     &                      read01,                ! Read unit #1
     &                      b_old,                 ! Already exists? (n/a).
     &                      code)                  ! Returned condition.
 
                do y=1,nrecord
 
                  if (data(y)(66:72).ne.'AUTOPIC'.and.
     &                data(y)(66:70).ne.'FINDP'.and.
     &                data(y)(76:78).ne.'ASI'.and.
     &                data(y)(76:78).ne.'API'.and.
     &                data(y)(76:78).ne.'FIP'
     &                ) then   ! added April 2001, lo
                    write(read01,'(a)') data(y)
                  endif
 
                enddo
 
                call sei close (close$,read01,code)
 
                endif


         if (new.or.picked) then

           if (auto_locate) then
              write(6,'(1x,a80)') evfile(1:80)
c
c  write seisan message to hyp to run hyp non interactive
c
             message = 'hyp non interactive'
             call put_seisan_message(message)

c
c loacte event
c
              call systemc('hyp',3)

c
c  update database first change 
c 

              call file_ready('hyp.out')

              call sei open(old$+warn$,            ! Open an old file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    'hyp.out',             ! File name
     &                    read01,                ! Read unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.

         call indata(read01,nstat,nphase,nhead,nrecord,type,exp,data,id)
              call sei close (close$,read01,code)

              do x=1,nhead
                if (data(x)(80:80).eq.'I') then
                  if (data(x)(9:11).eq.'NEW') data(x)(9:11)='HYP'
                endif
              enddo
c
c check if event above min magnitude for sending out email
c
              if (email_alert) then
                  mag=-99.
                  if (data(1)(57:59).ne.'   ') then
                    read(data(1)(57:59),'(f3.1)') mag
                  endif

                  if (mag.gt.min_mag) then

c
c write temp file with date and hypo.out
c
                  call file_ready('autopro.txt')
                  call sei open(unknown$,        ! Open an old file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    'autopro.txt',         ! File name
     &                    write01,               ! Read unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.
                  call file_ready('autopro_red.txt')
                  call sei open(unknown$,        ! Open an old file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    'autopro_red.txt',     ! File name
     &                    write02,               ! Read unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.

                  call gmttim(time)
                  write(write01,'(a)')
                  write(write01,'(2a)') ' ----- SEISNET AUTOPRO ----- ',
     &                  time
                  write(write01,'(a)')
                  write(write02,'(a)')
                  write(write02,'(2a)') ' ----- SEISNET AUTOPRO ----- ',
     &                  time
                  write(write02,'(a)')

c
c reduced version with hypocenter striped and P readings only
c
                  do x=1,nrecord
                    write(write01,'(a)') data(x)
                    if (x.eq.1) write(write02,'(a)') data(x)(1:20)
                    if (data(x)(80:80).eq.'7') then
                      write(write02,'(a)')data(x)
                    endif
                    if (x.gt.nhead.and.data(x)(11:11).eq.'P') then
                      write(write02,'(a)')data(x)
                    endif
                  enddo
                  write(alert_file,'(a)') time//' '//data(1)
                  call sei close (close$,write01,code)
                  call sei close (close$,write02,code)

c
c send email
c
                 line = ' '

                 if (email_address(1:19).ne.
     &             'alert@emsc-csem.org') then
                   line = 'mailx -s "'//data(1)(1:79)//'" '//
     &       email_address(1:seiclen(email_address))
     &             // ' < autopro.txt'
                    write(*,'(a)') line(1:seiclen(line))
                    call systemc(line,seiclen(line))
                  else
c
c if emsc first email, send reduced email to emsc, but full email to rest
c
                   line = 'mailx -s "'//data(1)(1:79)//'" '//
     &       email_address(1:19)
     &             // ' < autopro_red.txt'
                    write(*,'(a)') line(1:seiclen(line))
                    call systemc(line,seiclen(line))

                   line = 'mailx -s "'//data(1)(1:79)//'" '//
     &       email_address(21:seiclen(email_address))
     &             // ' < autopro.txt'
                    write(*,'(a)') line(1:seiclen(line))
                    call systemc(line,seiclen(line))
                  endif

c
c delete autopro.txt
c
                  call file_ready('autopro.txt')
                  call sei open(old$+warn$,                ! Open an old file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    'autopro.txt',         ! File name
     &                    read01,                ! Read unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)                  ! Returned condition.

                  call sei close (delete$,read01,code)
                endif
              endif

              call file_ready(evfile)
              call sei open(old$+warn$,            ! Open an old file.
     &                      ' ',                   ! Prompt file name (n/a).
     &                      evfile,                ! File name
     &                      read01,                ! Read unit #1
     &                      b_old,                 ! Already exists? (n/a).
     &                      code)                  ! Returned condition.

              do x=1,nrecord
                  write(read01,'(a)') data(x)
              enddo

              call sei close (close$,read01,code)


c           new = .false.

           endif 

         endif

c-- (CLK)
         else
c-- (CLK)
           write(6,'(1x,a80,a)') evfile(1:80),' doesn''t exist.'
c-- (CLK)
         endif
c-- back for next event
         goto 5
       else
c-- 3 is end of time period
          if(status.eq.2) write(6,*)' End of index file'
          if(status.gt.3)
     *    write(6,*)' ****** Something wrong, status= ', status
       endif

      call sei close (close$,alert_file,code)
      write(*,*) 'FINISHED'
 
      stop   
      end
 

