
c    Copyright 2003 Lars Ottemoller and Jens Havskov
c
c    This file is part of SEISNET.
c
c    SEISNET is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    Foobar is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with Foobar; if not, write to the Free Software
c    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USa

      PROGRAM FSEISNET

c updates:
c
c      04.12.97, lo : changed update_iris to use station name 
c      22.01.98  lo : modified update routines to check for time and date
c      01.09.98  lo : fixed bug in qnx conversion
c      05.09.98  lo : taken out phaselines
c      05.10.98  lo : move mag for quake to last pos
c      05.10.98  lo : use con insteas of par and wav in daily log
c
c   ---- Seisnet 2.0 ---- for Seisan 7.0
c      20.12.98  lo : include century in start and end_time
c      18.01.99  lo : add routine for creating new event
c      26.01.99  lo : added routines to handle autodrm
c                     new routines for writing lines (nordic,id,wave)
c      11.02.99  lo : changes for new iris type
c      20.03.99  lo : in routine add... also look for 2nd header and
c                     use fixed time, if no location in header
c      21.03.99  lo : use gmt time instead of system time
c      22.06.99  lo : fix bug with 5 char station code in qnx_cnordic
c      26.06.99  lo : fix bug in get_end_time
c      04.08.99  lo : fix output bug in autodrm
c      22.09.99  lo : added qnx to add_data routine
c      16.01.00  lo : give qnx parameter set to qnx_cnordic
c      10.02.00  lo : always use 'R' as distance indicator
c      25.02.00  lo : fix bug in extract
c      29.02.00  lo : fix bug with event type for log summary file
c      17.03.00  lo : put the stopbefore now option back into ftp_cnordic
c      09.05.00  lo : fix update of QNXEXT call
c      10.05.00  lo : fix bug in add_data, IRISB part
c      05.12.00  lo : dont put star into first line
c      19.02.01  lo : several smaller changes, done at the BGS
c      19.02.01  lo : bug fix in station_monitor routine
c      05.02.01  lo : move REQ in wave request line to end of line, and shorten
c                     replaced FINDP by ASI for autosig
c      17.06.01  lo : added naqs to add_data routine
c      21.11.01  lo : fix log_summary to handle path as part of log_file
c      22.11.01  lo : all subroutines moved out                          
c      10.04.03  lo : add call to update_seiext
c

c 
c     READ log.dat FILE AND WRITE LIST FILE OF DETECTIONS
c
c
      implicit none
c
      integer nars               ! number of arguments
      character*80  args(15)     ! arguments
      character*80  daylogfile   ! directory of day logs
      character*2   intim        ! input time string
      character*5   basename     ! data base
      character*5   station_name ! station name
      character*6   station_type ! station type (QNX, QUAKE, IRIS, SEISAN)
      character*14  endtime      ! end time to find las recorded event
      character*20  xtime        ! time string
      integer       interval     ! time interval
      character*70  eventfile    ! name of *.EVE file
      integer       backdays     ! look 'backdays' days in database
      character*40  last         ! last file in database
      character*255 sfile        ! name of S-file to update
      character*6   channel      ! IRIS component
      integer       ind          ! index
      character*40  wave_name    ! waveform filename
      character*40  wave_qnx     ! waveform filename on QNX in request line
      character*42  wave_ftp,    ! waveform filename on FTP in request line
     &              wave_name_ftp
      character*50  wave_ext
      character*40  wave_autodrm !        ...  autodrm ....
      character*8   time         ! time in waveform request
      character*10  date         ! date in waveform request
      character*80  log_file     ! path and file name of logfile
      character*80  html_log_dir ! path to store html log file       
      character*5   log_base     ! name of Log database
      character*8   input_date   ! current date
      character*17  start_time   ! start time
      character*14  stime        ! autodrm start time
      character*5   time_dif     ! time diff
      character*30  region       ! coordinates of region
      character*5   central_base ! name of central database
      character*3 type           ! PAR or WAV
      character*5 station        ! station name
      character*10 ldate
      character*75 path          ! path
      character*80 file          ! some file name
      character*80 parameter_file! name of SEISNET par file
      character*40 email         ! SEISNET's email address
      character*60 autodrm_file  ! autodrm file
      logical autodrm_check      ! true if autodrm file
      character*30 refid         ! reference ID
      character*30 datatype      ! message ID
      integer c                  ! counter
      character*40 filename_pattern ! pattern of file names
      character*8 autopick       ! FINDP or AUTOPIC
      character*1 par_set        ! number of seislog parameter set
      integer stopsecbefore      ! used to give stop time before now
      integer seiclen
      character*10 autodrm_agency ! agency for autodrm request
      integer ext_stat_index
      character*(80) ext_channel,ext_quake_stat
      character*(240) ext_quake_chan
      character*(14) ext_time
      character*20 naqs_channel
      real ext_duration,ext_pre
      character*80 naqs_log_file ! file name of naqs log file
      character*10 naqs_pre,naqs_post
      integer det_wav_option ! 2 digit integer 0=true, 1=false,
                             ! 1st is flag for event detection, 
                             ! 2nd for waveform transfer
      character*240 event_dir    ! event detection directory
      character*240 wave_req_dir ! waveform request directory
      character*10 sdasdate,sdastime ! time of sdas request
      character*5 sdasstation

c
c version
c
      call print_ver 
     & ('SEISNET - fseisnet - version: 2.0.5   date: 06/04/2000')



  1   CONTINUE 
      nars=0
      call get_arguments(nars,args)

      if (nars.ne.0) THEN
        if (args(1).EQ.'makelist') THEN
c commented out, Nov 2001, lo
c          station_name=args(2)(1:5)
c          daylogfile=args(3)
c          call make_event_list(station_name,daylogfile)
c        ELSEIF (args(1).EQ.'dirf') THEN
c          eventfile=args(2)
c          call dirf(eventfile) 
        ELSEIF (args(1).EQ.'autodrm') THEN
          station_name=args(2)(1:5)
          basename=args(3)
          email=args(4)
          stime=args(5)
          autodrm_agency=args(6)
          parameter_file=args(7)
          call autodrm(station_name,basename,email,stime,
     &       autodrm_agency,parameter_file)
        ELSEIF (args(1).eq.'autodrmin') then
          autodrm_file=args(2)
c          basename=args(3)
          autodrm_check=.false.
          call check_autodrm_file(autodrm_file,autodrm_check,refid,
     &       datatype)
          if (refid.ne.'(No_REF_ID_received)') then
c            call find_autodrm_sfile(refid,basename)
            call find_autodrm_sfile(refid,datatype)
          else
            write(*,*) 'NO REFID'
          endif
c        ELSEIF (args(1).eq.'CTIME') then
c          intim(1:2)=args(2)(1:2)
c          call c_time(intim)
        ELSEIF (args(1).eq.'CNORD') then
          station_name=args(2)
          station_type=args(3)
          basename= args(4)

          do c=2,5
            if (basename(c:c) .eq.' ') basename(c:c)='_'
          enddo
          endtime= args(5)
          read(args(6),'(i6)') backdays
          call find_last(last,basename,endtime,backdays,
     *                      station_type)
          if (station_type(1:3).eq.'QNX') then
             par_set = args(7)
             event_dir = args(8)
             wave_req_dir = args(9)
             read(args(10),'(i2)') det_wav_option
             call qnx_cnordic(station_name,basename,last,
     &            par_set,event_dir,wave_req_dir,det_wav_option)
          elseif (station_type.eq.'SDAS') then
             event_dir = args(7)
             wave_req_dir = args(8)
             read(args(9),'(i2)') det_wav_option
             read(args(10),'(f5.0)')ext_pre
             read(args(11),'(f5.0)')ext_duration
             par_set = args(12)
             call sdas_cnordic(station_name,basename,last,
     &            event_dir,wave_req_dir,det_wav_option,
     &            ext_pre,ext_duration,par_set)
          elseif (station_type.eq.'SEISAN') then
             sfile=args(7)   ! Nordic input file
             event_dir = args(8)
             wave_req_dir = args(9)
             read(args(10),'(i2)') det_wav_option
             call seisan_cnordic(station_name,basename,sfile,
     &            event_dir,wave_req_dir,det_wav_option)
          elseif(station_type(1:3).eq.'VME') then
             event_dir = args(7)
             wave_req_dir = args(8)
             read(args(9),'(i2)') det_wav_option
             call vmeseislog_cnordic(station_name,basename,last,
     &          event_dir,wave_req_dir,det_wav_option)
          elseif (station_type.eq.'IRISA'.or.
     &             station_type.eq.'IRISB') then
             event_dir = args(7)
             wave_req_dir = args(8)
             read(args(9),'(i2)') det_wav_option
             call iris_cnordic(station_name,station_type,
     &          basename,last,event_dir,wave_req_dir,det_wav_option)
          elseif (station_type.eq.'NAQS') then
             naqs_log_file=args(7)
             naqs_channel=args(8)
             naqs_pre=args(9)
             naqs_post=args(10)
             event_dir = args(11)
             wave_req_dir = args(12)
             read(args(13),'(i2)') det_wav_option
             call naqs_cnordic(station_name,basename,last,naqs_log_file,
     &               naqs_channel,naqs_pre,naqs_post,
     &               event_dir,wave_req_dir,det_wav_option)
          elseif (station_type.eq.'QUAKE') then
            region=args(7)
            central_base=args(8)              
            ext_quake_stat=args(9) 
            ext_quake_chan=args(10)
            read(args(11),'(f6.0)') ext_duration
            parameter_file=args(12)
            wave_req_dir = args(13)
            call quake_cnordic(station_name,basename,last,region,
     *         central_base,ext_quake_stat,ext_quake_chan,ext_duration,
     *         parameter_file,wave_req_dir)
          elseif (station_type.eq.'FTP') then
            parameter_file = args(7)
            read(args(8),'(i6)') stopsecbefore
            event_dir = args(9)
            wave_req_dir = args(10)
            read(args(11),'(i2)') det_wav_option
            call ftp_cnordic(station_name,basename,last,
     &           parameter_file,stopsecbefore,
     &           event_dir,wave_req_dir,det_wav_option)
          endif
        ELSEIF (args(1).eq.'LASTE') then
          station_type= args(3)
          basename= args(4)
          endtime= args(5)
          read(args(6),'(i6)') backdays
          call find_last(last,basename,endtime,backdays,
     *                      station_type)
        ELSEIF (args(1).eq.'HOURB') then
          station_type= args(3)
          basename= args(4)
          endtime= args(5)
          read(args(6),'(i6)') backdays
          call hours_back(basename,endtime,backdays,
     *                      station_type)
        ELSEIF (args(1).eq.'ADDTI') then
          xtime= args(2)
          read(args(3),'(i6)') interval
          call add_time(xtime,interval)
          
        ELSEIF (args(1).eq.'UPDAT') then
          station_type=args(2)
          sfile= args(3)
          autopick=args(4)

          if (station_type(1:6).eq.'QNXEXT') then
            station_name=args(5)
            wave_qnx =args(6)
            do c=1,17
              if (wave_qnx(c:c).eq.'_') wave_qnx(c:c)=' '
            enddo
            wave_ext=args(7)
            wave_req_dir=args(8)

            call update_qnxext(sfile,station_name,wave_qnx,wave_ext,
     &              autopick,wave_req_dir)
          elseif (station_type(1:3).eq.'QNX') then
            wave_qnx =args(5)
            wave_name=args(6)
            call update_qnx(sfile,wave_qnx,wave_name,autopick)

          elseif (station_type(1:6).eq.'SEISAN') then
            wave_name=args(5)
            call update_seisan(sfile,autopick,wave_name)
          elseif (station_type(1:6).eq.'SEIEXT') then
            station_name=args(5)
            wave_qnx =args(6)
            wave_ext=args(7)
            wave_req_dir=args(8)

            call update_seiext(sfile,station_name,wave_qnx,wave_ext,
     &              wave_req_dir)
          elseif (station_type(1:4).eq.'SDAS') then
            sdasdate=args(5)
            sdastime=args(6)
            sdasstation=args(7)
            wave_name=args(8)
            call update_sdas(sfile,sdasdate,sdastime,sdasstation,
     &            wave_name,autopick)

          elseif (station_type(1:3).eq.'VME') then
            wave_qnx =args(5)
            wave_name=args(6)
            call update_vmeseislog(sfile,wave_qnx,wave_name,autopick)

          elseif (station_type.eq.'FTP') then

            wave_ftp =args(5)
            wave_name_ftp =args(6)

            call update_ftp(sfile,wave_ftp,wave_name_ftp,autopick)

          elseif (station_type.eq.'IRIS') then
 
            date = args(5)
            time = args(6)
c         write(*,*) date,' ',time
            wave_name=args(7)
c
c   merge channel from two args
c
 
            station_name=args(8)
            channel=args(9)
c
c should not happen anymore ..., bug fixed
c
              if (nars.eq.10) then
                ind = seiclen(channel)
                channel=channel(:ind) // args(10)(1:5-ind)
              endif

c              write(*,*) channel
   
             call update_iris(sfile,date,time,wave_name,station_name,
     &       channel,autopick)

          elseif (station_type.eq.'AUTODR') then

            wave_autodrm =args(5)
            refid        =args(6)

            call update_autodrm(sfile,wave_autodrm,refid,autopick)

          elseif (station_type.eq.'NAQS') then
            xtime        =args(5)
            station_name =args(6)
            naqs_channel =args(7) 
            wave_name    =args(8)
            call update_naqs(sfile,xtime,station_name,
     &          naqs_channel,autopick,wave_name)
          endif

        ELSEIF (args(1).eq.'LOGSU') then
          log_file=args(2)
          log_base=args(3)
          html_log_dir=args(4)

          do c=2,5
            if (log_base(c:c).eq.' ') log_base(c:c)='_'
          enddo

          call log_summary(log_file,log_base,html_log_dir)

        ELSEIF (args(1).eq.'MONIT') then

          path=args(2)
          type=args(3)
          station=args(4)
          ldate=args(5)
          time=args(6) 
 
          call station_monitor(path,type,station,ldate,time)

        ELSEIF (args(1)(1:3).eq.'CAT') then
 
          file=args(2)
 
          call cat_file(file)

        ELSEIF (args(1).eq.'PREDA') then

          input_date=args(2)

          call previous_day(input_date)

        ELSEIF (args(1).eq.'ENDTI') then

          start_time=args(2)
          time_dif  =args(3)
          call get_end_time (start_time,time_dif)

        ELSEIF (args(1).eq.'ADDDA') then

          parameter_file=args(2)
          central_base  =args(3)
          sfile         =args(4)
          wave_req_dir  =args(5)

          if (nars.gt.6) then
            read(args(6),'(i3)') ext_stat_index
            ext_channel = args(7)
            ext_time = args(8)
            read(args(9),'(f6.0)') ext_duration
          else
            ext_stat_index=0
            ext_channel = ' '
            ext_time = ' '
            ext_duration = 0.
          endif
          ext_pre=0.
          
          call add_data(parameter_file,central_base,sfile,
     &      ext_stat_index,ext_channel,ext_time,ext_duration,ext_pre,
     &       wave_req_dir) 

        ELSEIF (args(1).eq.'FREADY') then
          
          file=args(2)
          write(*,*) ' checking file ' // file(1:seiclen(file))
          call file_ready(file(1:seiclen(file)))
        ELSE
          write(*,*) 'wrong choice !!! '
        ENDIF
      ELSE
        write(*,*) 'wrong choice !!! '
      ENDIF

 9999 CONTINUE
  
      write(*,'(a)') 'FINISHED'

      stop
        
      END



