
c    Copyright 2003 Lars Ottemoller and Jens Havskov
c
c    This file is part of SEISNET.
c
c    SEISNET is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 2 of the License, or
c    (at your option) any later version.
c
c    Foobar is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with Foobar; if not, write to the Free Software
c    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USa

c
c This file contains SEISNET FORTRAN subroutines that are used for Nanometrics NAQS
c type stations.
c

c
c List of subroutine in alphabetical order:
c
c   naqs_channel_select - read component codes from seisnet parameter file 
c   naqs_cnordic - convert trigger file to nordic format
c   naqs_update - update s-file after waveform transfer

c
c Changes:
c
c  28 Nov 2001 lo: added documentation and clean-up
c

      subroutine naqs_channel_select(pfile,station_name,chan_list,nchan)
c
c read component codes from seisnet parameter file 
c
      implicit none

      character*5 station_name,station
      character*35 chan_list(*)
      integer nchan
      character*(*) pfile
      
      include 'seidim.inc'
      include 'libsei.inc'                ! Open file definitions
      external sei open,                  ! Open file routine.
     &         sei close,                 ! Close file routine.
     &         sei code                   ! Error encoder.
      integer read01,code
      logical b_old
      integer i,j,x,ind
      character*80 par_line

      nchan=0
      x=-1
      
      call file_ready(pfile)

      call sei open(old$+warn$,                  ! Open a unknown status file.
     &                    ' ',                   ! Prompt file name (n/a).
     &                    pfile,                 ! File name
     &                    read01 ,               ! Write unit #1
     &                    b_old,                 ! Already exists? (n/a).
     &                    code)

10    continue
      read(read01,'(a)',END=15) par_line

c
c find station in file
c
      if (par_line(5:12).eq.'STATION(') then
         ind=index(par_line,')')
         read(par_line(31:35),'(a5)') station
         if (station.eq.station_name) then
           read(par_line(13:ind-1),'(i3)') x
         endif
c
c  find components for station name
c
      elseif (par_line(5:14).eq.'COMPONENTS') then

        ind=index(par_line,')')

        read(par_line(16:ind-1),'(i3)') i

        if (i.eq.x) then
          nchan=nchan+1
cet COMPONENTS(1)             "*.*                 12345     12345"
          read(par_line(32:66),'(a)') chan_list(nchan)

        endif

      endif


      goto 10
15    continue

c
c close parameter file
c
      call sei close (close$,read01,code)

c
c  output
c
      if (x.eq.-1) then
         write(*,*) 'station not in list !!!'
         return
      else
        write(*,*) station_name,' found ...'
        do x=1,nchan
          write(*,*) chan_list(x)
        enddo
      endif

      return
      end


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc



      subroutine naqs_cnordic(station,base,lastevent,
     &  infile,channel,cpre,cpost,event_dir,wave_req_dir,det_wav_option)
c
c convert trigger file to nordic format
c

      implicit none

      include 'seidim.inc'
      integer  sei clen      ! character length
      character*(*) station
      character*(*) base
      character*(*) lastevent
      character*(*) infile
      character*(*) channel
      character*(*) cpre,cpost
      integer nevent,i             ! counter
      real pre,post
      character*120 line
c -- date and time of last event
      integer last_year,last_month,last_day
      integer last_hour,last_minute,last_sec
      character*14 currentevent
      integer year,month,day,hour,min,sec
      real rsec
      double precision msec
      character*80 waveline,idline,nordicline
      character*80 sfile
      integer length
      integer status
      character*50 naqs_wav_text
c option for waveform transfer
      integer det_wav_option
c directories for network event detection and waveform transfer
      character*(*) event_dir,wave_req_dir

      nevent=0
      read(cpre,'(f6.0)') pre
      read(cpost,'(f6.0)') post

c
c get date and time of last event
c
      read(lastevent(1:4),'(i4)') last_year
      read(lastevent(5:6),'(i2)') last_month
      read(lastevent(7:8),'(i2)') last_day
      read(lastevent(9:10),'(i2)') last_hour
      read(lastevent(11:12),'(i2)') last_minute
      read(lastevent(13:14),'(i2)') last_sec

c
c open input file
c
      call file_ready(infile)
      open(50,file=infile,status='old')

10    continue

      read(50,'(a)',end=999) line
c2345678901234567890123456789012345678901234567890123456789012345678901234567890
c 2001-06-26 12:04:47 NMX-A  EventFactory....(1) Event detected: 1 08:04:33.988

c
c event declared by the naqs
c
      if (line(30:41).eq.'EventFactory') then

c        write(*,*) line(3:21)
        read(line(3:21),'(i4,5(1x,i2))') year,month,day,hour,min,sec
        rsec=float(sec)

        write(currentevent,'(i4,5(i2))') year,month,day,hour,
     &        min,sec
        do i=1,14
           if (currentevent(i:i).eq.' ') then
             currentevent(i:i)= '0'
           endif
        enddo
        call compare(currentevent,lastevent,status)
        if (status.le.0) goto 10

        call write_nordicline(nordicline,year,month,day,hour,min,rsec,
     &        'R',0.,0.,0.,' ',station(1:3),1)        
        call write_idline(idline,year,month,day,hour,min,sec)
        naqs_wav_text=
     &  '                                                       '
c
c all channels for waveform transfer
c
        naqs_wav_text(1:50)=
     &   '                                                  '
        write(naqs_wav_text(1:50),'(i4.4,5i2.2,1x,a20,i5,5x,i5)')
     &          year,month,day,
     &          hour,min,sec,
c     &          '*.*                 ',
     &          channel,
     &          int(pre),int(post)
        call write_waveline(waveline,'EXT',station,'NAQ',
     &          naqs_wav_text)

        call sfilname(year,month,day,hour,
     &      min,sec,base,'R',sfile,length )
        call file_ready(sfile)

        open(60,file=sfile,status='new')
c
c output
c
        write(*,'(a)') nordicline
        write(60,'(a80)') nordicline
        write(60,'(a80)') idline
        write(60,'(a80)') waveline
        write(60,'(2a40)') ' STAT SP IPHASW D HRMM SECON CODA AMPLIT',
     &     ' PERI AZIMU VELO SNR AR TRES W  DIS CAZ7'
        write(60,*)

        close(60)

        call copy_det_file(sfile,event_dir,wave_req_dir,det_wav_option)

        nevent=nevent + 1

      endif

      
      goto 10 
999   continue
      close(50)
      write(*,*) '... in total ',nevent,' detections ...'


      return
      end



      subroutine update_naqs(sfile,xtime,station_name,
     &          naqs_channel,autopick,wave_name)
c
c update s-file after waveform transfer
c
      implicit none

      include 'libsei.inc'                ! Open file definitions
      include 'seidim.inc'

      character*(*) autopick              ! name of autopick routine
      character*(*) sfile                 ! name of S-file
      character*(*) xtime
      character*14  time
      character*5   station_name      
      character*(*) naqs_channel
      character*(*) wave_name

      logical      there                ! test if file exists

c-- event data
       character*80     data(max_data)
c-- exp indicator
       character*1      exp
c-- event type
       character*1      type
c-- number of recors for event
       integer          nrecord
c-- number of header lines
       integer          nhead
c number of phases
       integer nphase
c read unit
       integer read01
c number of stations
       integer nstat
c-- id line number
       integer id
c counter
      integer x
c waveform line
      character*80 waveline
c autopick line
      character*80 autopick_line
c flag if autopick line
      logical flag
c index
      integer ind
c line with 'REQUEST'
      integer requ_line
c id
      character*14 idline
      integer seiclen


      requ_line = -1

c
c   check if file exists
c

      inquire (file=Sfile,exist=there)
      if (.NOT.THERE) then
         write(*,*) Sfile,' does not exist !'
         return
      endif

c
c  open S-file
c
      call file_ready(Sfile)
      open(unit=8,file=Sfile,status='old')

c
c read event file
c
      read01=8
      call indata(read01,nstat,nphase,nhead,nrecord,type,exp,data,id)

      close(8)

      time = '              '
c23456789012345678
c999-01-25-10-09-10
      time(1:4) = xtime(1:4)
      time(5:6) = xtime(6:7)
      time(7:8) = xtime(9:10)
      time(9:10) = xtime(12:13)
      time(11:12) = xtime(15:16)
      time(13:14) = xtime(18:19)


c
c look for "REQUEST"
cNET WAV EXT NAQS1NAQ 20010614120000 *.*                 10        100     REQ 3
c
      do x=1,nhead
        if (data(x)(66:72).eq.'REQUEST'.or.
     &      data(x)(76:78).eq.'REQ') then
c       write(*,*) ' xxx ',
c     &     data(x)(38:37+(seiclen(naqs_channel))),' ',
c     &     naqs_channel(1:seiclen(naqs_channel))
c      if (data(x)(38:37+(seiclen(naqs_channel))).ne.
c     &     naqs_channel(1:seiclen(naqs_channel))) 
c     &   write(*,*) ' xxx wrong comp '
        if (data(x)(23:36).ne.time) write(*,*) ' time'
        if (station_name.ne.data(x)(14:18)) write(*,*) ' stat'
          if (data(x)(14:18).eq.station_name.and.
c     &        data(x)(23:36).eq.time.and.
     &        data(x)(38:37+(seiclen(naqs_channel))).
     &          eq.naqs_channel(1:seiclen(naqs_channel))) then
            requ_line=x
         write(*,*) ' request line ',x
          endif
        endif
      enddo

c
c  make autopick_line
c
cNET WAV EVE TRO  FTP 9809_05_0000_58T.TRO_03_1        STA050534 AUTOPICK

      if (requ_line.gt.0) then

      call write_autopick_line(data(requ_line),autopick_line,
     &    wave_name,autopick,flag)

c
c  put transferred
c
c        data(requ_line)(66:76)='TRANSFERRED'
        data(requ_line)(76:78) = 'TRA'   ! changed April 2001, lo
      endif


c
c blank new line
c
        do x=1,80
           waveline(x:x) = ' '
        enddo
c
c write info to new line
c
        write(waveline(2:41),'(a40)') wave_name
        write(waveline(80:80),'(a1)') '6'
c
c update according to status of file transfer
c

        call file_ready(Sfile)
        open(8,file=Sfile,status='unknown')

        do x=1,nhead-1
           write(8,'(a80)') data(x)
        enddo


        if (flag) write(8,'(a80)') autopick_line

        write(8,'(a80)') waveline

        do x=nhead,nrecord
          write(8,'(a80)') data(x)
        enddo
c
c close updated S-file and end process
c

        close(8)


      return
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


 
 
